// SUDoc.cpp : implementation of the CSUDoc class
//
//
// SyncURLs version 1.0
// Copyright 1998 Ziff-Davis Inc.
// First published in PC Magazine, US Edition
// January 6, 1998
// Written by Gregory A. Wolking
//

#include "stdafx.h"
#include <intshcut.h>
#include <direct.h>
#include <errno.h>

#include "SyncURLs.h"
#include "StatDlg.h"
#include "BadDlg.h"
#include "MyList.h"
#include "WeedDlg.h"
#include "SUDoc.h"
#include "BrowsDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

// OLE interface pointers.
//IUniformResourceLocator* pIURL;
//IPersistFile* pIPF;
IMalloc* pMalloc;
CStatDlg* pStatDlg;

/////////////////////////////////////////////////////////////////////////////
// CSUDoc

IMPLEMENT_DYNCREATE(CSUDoc, CDocument)

BEGIN_MESSAGE_MAP(CSUDoc, CDocument)
	//{{AFX_MSG_MAP(CSUDoc)
	ON_UPDATE_COMMAND_UI(ID_FILE_SAVE, OnUpdateFileSave)
	ON_UPDATE_COMMAND_UI(ID_FILE_SAVE_AS, OnUpdateFileSaveAs)
	ON_UPDATE_COMMAND_UI(ID_EXPORT_NS, OnUpdateExportNs)
	ON_COMMAND(ID_EXPORT_TEXT, OnExportText)
	ON_UPDATE_COMMAND_UI(ID_EXPORT_TEXT, OnUpdateExportText)
	ON_UPDATE_COMMAND_UI(ID_EXPORT_MSIE, OnUpdateExportMsie)
	ON_UPDATE_COMMAND_UI(ID_EXPORT_BOTH, OnUpdateExportBoth)
	ON_COMMAND(ID_FILE_MANUAL, OnFileManual)
	ON_UPDATE_COMMAND_UI(ID_EXPORT_MANUAL, OnUpdateExportManual)
	ON_UPDATE_COMMAND_UI(ID_EXPORT_NSC, OnUpdateExportNsc)
	ON_COMMAND(ID_VIEW_LOG, OnViewLog)
	ON_UPDATE_COMMAND_UI(ID_VIEW_LOG, OnUpdateViewLog)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CSUDoc construction/destruction

CSUDoc::CSUDoc()
{
	CWinApp* pApp = AfxGetApp();
	CString line1, line2;
	int pos;

	m_intItemCount = 0;
	m_pDocElements = NULL;
	m_intPathCount = 0;
	m_pPathList = NULL;
	m_pErrFile = NULL;
	m_pStatDlg = NULL;
	m_bNSPresent = FALSE;
	m_bIEPresent = FALSE;
	m_bNSCPresent = FALSE;
	m_strIEPath.Empty();
	m_strNSPath.Empty();
	m_strNSCUserName.Empty();
	m_strNSCPath.Empty();
	m_strNSPageTitle = "Bookmarks Exported by SyncURLs";
	m_strNSCPageTitle = "Bookmarks Exported by SyncURLs";
	pStatDlg = NULL;
	// Get filename for import log.
	line1 = pApp->m_pszHelpFilePath;
	pos = line1.ReverseFind('\\') + 1;
	line2 = line1.Left(pos);
	line2 += "IMPLOG.TXT";
	m_strErrLog = line2;
	// Set Browser-related variables.
	Detect_Browsers();
	// Initialize IMalloc interface pointer.
	if (SHGetMalloc(&pMalloc) == E_FAIL)
	{
		pMalloc = NULL;
		AfxMessageBox("SHGetMalloc failed.", MB_ICONSTOP);
	}
}

CSUDoc::~CSUDoc()
{
	if (pMalloc)
		pMalloc->Release();
}

/////////////////////////////////
// Standard Document function overrides

// Called at app startup, or when File/New command is issued.
BOOL CSUDoc::OnNewDocument()
{
	if (!CDocument::OnNewDocument())
		return FALSE;

	// Cannot continue if constructor could not initialize OLE interface pointers.
	if (/*pIURL == NULL || pIPF == NULL ||*/ pMalloc == NULL)
		return FALSE;

	// Cannot continue if neither browser was detected.
	if (!(m_bNSPresent || m_bIEPresent))
		AfxMessageBox("Could not detect either MSIE or Netscape!", MB_ICONSTOP);
	else
		// Must re-import to ensure that tree structure is in synch
		// with the Favorites folder.
		Do_Import();
	return TRUE;		// Report success.
}

BOOL CSUDoc::OnOpenDocument(LPCTSTR lpszPathName) 
{
	CString msg;

	m_bLoadFailed = FALSE;				// Clear error flags. These will be set appropriately
	m_bInternalLoadFail = FALSE;		// by CSUDoc::Serialize if the file couldn't be loaded.
	m_bGotSomeData = FALSE;
	if (!CDocument::OnOpenDocument(lpszPathName))
		return FALSE;
	if (m_bLoadFailed)
	{
		if (m_bInternalLoadFail)
			msg.Format("Internal error occurred while\n"
						"reading %s.\n\n", lpszPathName);
		else
			msg.Format("Error reading %s\n"
				"The file is either not a Bookmark Library,\n"
				"or it is damaged in some way.\n\n", lpszPathName);
		AfxMessageBox(msg, MB_ICONINFORMATION);
		if (m_bGotSomeData)
			AfxMessageBox(IDP_PARTIAL_LOAD_FAIL, MB_HELP | MB_ICONINFORMATION);
		else
			AfxMessageBox(IDP_TOTAL_LOAD_FAIL, MB_ICONINFORMATION);
	}
	Do_Import();	
	return TRUE;
}

// Called as part of File/New and File/Open
void CSUDoc::DeleteContents() 
{
	UINT i;

	if (m_intItemCount)								// If element array is not empty,
	{
		// Serious problem if array pointer is already NULL!
		ASSERT(m_pDocElements != NULL);

		for (i=0; i < m_intItemCount; i++)			// For each element,
		{
			if (m_pDocElements[i].page)				// Delete page name string.
				delete m_pDocElements[i].page;
			if (m_pDocElements[i].url)				// Delete URL string.
				delete m_pDocElements[i].url;
			if (m_pDocElements[i].comment)			// Delete comment string.
				delete m_pDocElements[i].comment;
		}
		delete [] m_pDocElements;					// Delete the array.
		m_pDocElements = NULL;						// NULL the pointer.
		m_intItemCount = 0;							// Reset item count.
	}
	if (m_intPathCount)								// If path array is not empty,
	{
		// Serious problem if array pointer is already NULL!
		ASSERT(m_pPathList != NULL);

		for (i=0; i < m_intPathCount; i++)			// For each array element,
		{
			if (m_pPathList[i].path)				// Delete the path string.
				delete m_pPathList[i].path;
		}
		delete [] m_pPathList;						// Delete the path array.
		m_pPathList = NULL;							// NULL the array pointer.
		m_intPathCount = 0;							// Reset the path count.
	}
}


/////////////////////////////////////////////////////////////////////////////
// CSUDoc serialization

void CSUDoc::Serialize(CArchive& ar)
{
	UINT i;
	CString blank("");

	if (ar.IsStoring())
	{
		ar << m_intPathCount;						// Write number of paths.
		if (m_intPathCount > 0)						// Write path strings.
		{
			for (i = 0; i < m_intPathCount; i++)
				ar << *m_pPathList[i].path;
		}
		ar << m_intItemCount;						// Write # of bookmarks.
		if (m_intItemCount > 0)						// Write each bookmark:
		{
			for (i = 0; i < m_intItemCount; i++)
			{
				ar << *m_pDocElements[i].page;		// Write page name string.
				if (m_pDocElements[i].url)			// If URL pointer is not NULL,
					ar << *m_pDocElements[i].url;	// write the comment,
				else								// otherwise
					ar << blank;					// write empty string.
				if (m_pDocElements[i].comment)		// Do the same for comment string.
					ar << *m_pDocElements[i].comment;
				else
					ar << blank;
				ar << m_pDocElements[i].path_offset;	// Write numeric fields.
				ar << m_pDocElements[i].add_date;
				ar << m_pDocElements[i].mod_date;
				ar << m_pDocElements[i].visit_date;
			}
		}
	}
	else
	{
		UINT path_count, item_count, offset;
		CString page, url, comment;
		LONG add_date, mod_date, visit_date;
		UINT * new_offsets = NULL;

		try
		{
			ar >> path_count;						// Get number of paths.
			new_offsets = new UINT[path_count];		// Create array of offsets.
			if (new_offsets == NULL)
				throw ((int) 3);
			for (i=0; i < path_count; i++)			// For each path:
			{
				ar >> page;							// Read the path string from the file.
				if(!Add_Path(page, FALSE, &new_offsets[i]))	// Add to array and store its new offset.
					throw ((int) 1);
				else
					m_bGotSomeData = TRUE;
			}
			ar >> item_count;						// Get number of items.
			for (i=0; i < item_count; i++)			// For each item,
			{
				ar >> page;							// read page string
				ar >> url;							// read page string
				ar >> comment;
				ar >> offset;						// read old page offset.
				ar >> add_date;
				ar >> mod_date;
				ar >> visit_date;
				if (!Add_Item(new_offsets[offset], page, url,
					comment.IsEmpty() ? NULL : (LPCTSTR) comment,
					add_date, mod_date, visit_date, NULL))
					throw ((int) 1);	// Add to array using new path offset.
				else
					m_bGotSomeData = TRUE;
			}
		}
		catch (int)
		{
			m_bLoadFailed = TRUE;
			m_bInternalLoadFail = TRUE;
		}
		catch (...)
		{
			m_bLoadFailed = TRUE;
		}
		if (new_offsets != NULL)
			delete [] new_offsets;					// Delete array of new offsets.
	}
}

/////////////////////////////////////////////////////////////////////////////
// CSUDoc diagnostics

#ifdef _DEBUG
void CSUDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CSUDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CSUDoc commands and UpdateUI handlers.
void CSUDoc::OnUpdateFileSave(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_intItemCount > 0);	
}

void CSUDoc::OnUpdateFileSaveAs(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_intItemCount > 0);
}


void CSUDoc::OnUpdateExportNs(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_intItemCount > 0 && m_bNSPresent);
}

void CSUDoc::OnExportText() 
{
	if (!Do_Export_Warning())
		return;
	Export_Text();
}

void CSUDoc::OnUpdateExportText(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_intItemCount > 0);	
}

void CSUDoc::OnUpdateExportMsie(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_intItemCount > 0 && m_bIEPresent);
}


void CSUDoc::OnUpdateExportBoth(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_intItemCount > 0 && ((m_bNSPresent + m_bNSCPresent + m_bIEPresent) > 1));
}


// "File" menu, "Manual Import" command.
void CSUDoc::OnFileManual() 
{
	CString strOldName;
	BOOL fatal = FALSE;

	CFileDialog dlg(TRUE, ".htm", "BOOKMARK.HTM", OFN_HIDEREADONLY, "HTML files (*.htm; *.html)|*.htm; *.html||", NULL);
	if (dlg.DoModal() != IDOK)
		return;
	if (!Start_Error_Log())
	{
		AfxMessageBox(IDP_ERR_LOG_FAIL);
		return;
	}
	strOldName = m_strNSPath;
	m_strNSPath = dlg.GetPathName();
	m_pStatDlg = new CStatDlg;
	m_pStatDlg->CenterWindow();
	m_pStatDlg->ShowWindow(SW_SHOW);
	m_pStatDlg->Show_Message("Importing Netscape bookmarks");
	m_pErrFile->WriteString("Beginning Netscape import\n\n");
	if (Parse_NS_File() == IDP_IMPORT_FAIL)
		fatal = TRUE;
	if (!m_bErrorsLogged)
		m_pErrFile->WriteString("Netscape import complete ... no errors.\n\n");
	Do_Import_Cleanup();
	SetModifiedFlag();
	m_strNSPath = strOldName;
	End_Error_Log();
	m_pStatDlg->DestroyWindow();
	delete m_pStatDlg;
	m_pStatDlg = NULL;
	if (m_bErrorsLogged)
		Query_Show_Log(fatal);
	UpdateAllViews(NULL);
}

/////////////////////////////////
// CSUDoc Helpers

// Adds a new bookmark to the document.
// If matching bookmark (same folder, page name, and url) exists,
// returns offset of existing item.
BOOL CSUDoc::Add_Item(UINT path_offset, LPCTSTR page, LPCTSTR url, LPCTSTR comment, LONG add_date, LONG mod_date, LONG visit_date, UINT * offset)
{
	DOC_ELEMENT *temp = NULL;
	DOC_ELEMENT new_element = {0, NULL, NULL};
	CString strTemp1, strTemp2;
	int len1, len2;
	char chTrail;

	ASSERT(page != NULL);	// Page name pointer must not be NULL!
	ASSERT(url != NULL);

	if (m_intItemCount)		// If list is not empty and URL is specified,
	{
		for (UINT i = 0; i < m_intItemCount; i++)	// See if matching entry exists
		{
			if (m_pDocElements[i].path_offset != path_offset)			// Must be in same folder.
				continue;
			if (m_pDocElements[i].page->CompareNoCase(page) != 0)		// With same page name
				continue;
			if (m_pDocElements[i].url->Compare(url) == 0)					// And same URL
			{
				if (offset)				// If requested,
					*offset = i;		// return item offset to caller.
				return TRUE;			// Indicate success.
			}
			else
			{
				// Check for case where URLs are identical except for
				// a trailing slash character
				len1 = m_pDocElements[i].url->GetLength();		// Length of existing URL
				len2 = strlen(url);								// Length of new URL
				if (abs(len1 - len2) == 1)						// If difference is one character,
				{
					if (len1 > len2)							// If existing URL is longer,
					{
						chTrail = m_pDocElements[i].url->GetAt(len2);	// Get trailing character from existing
						strTemp1 = m_pDocElements[i].url->Left(len2);	// USL, then strip it.
						strTemp2 = url;									// Copy new URL verbatim.
					}
					else										// If existing URL is shorter,
					{
						chTrail = url[len1];					// Get trailing char from new URL.
						strTemp1 = url;							// Strip it.
						strTemp2 = strTemp1.Left(len1);
						strTemp1 = *m_pDocElements[i].url;		// Copy existing URL.
					}
					// If the extra character was a slash and the URLs match,
					if ((chTrail == '/') && (strTemp1 == strTemp2))
					{
						if (offset)								// If requested,
							*offset = i;						// return offset to caller.
						return TRUE;							// Indicate success.
					}
				}
			}
		}
	}
	new_element.path_offset = path_offset;		// Save offset to path string.
	new_element.page = new CString(page);		// Create string for page name.
	if (new_element.page == NULL)				// If creation failed,
		return FALSE;							// Report error.
	new_element.url = new CString(url);		// Create new CString for url.
	if (new_element.url == NULL)			// If creation failed,
	{
		delete new_element.page;			// delete string for page name
		return FALSE;						// and report error.
	}
	if (comment == NULL)						// If comment pointer is NULL,
		new_element.comment = NULL;				// just copy it.
	else												// Otherwise,
	{
		new_element.comment = new CString(comment);		// Create new CString for comment.
		if (new_element.comment == NULL)				// If creation failed,
		{
			if (new_element.url)						// if URL string exists,
				delete new_element.url;					// delete it.
			delete new_element.page;					// delete page name string.
			return FALSE;								// Report failure.
		}
	}
	new_element.add_date = add_date;
	new_element.mod_date = mod_date;
	new_element.visit_date = visit_date;
	// If element array doesn't exist yet,
	if (m_intItemCount == 0)
		// Create it.
		temp = (DOC_ELEMENT *) malloc(sizeof(DOC_ELEMENT));
	else	// Otherwise,
		// Try to expand it.
		temp = (DOC_ELEMENT *) realloc(m_pDocElements, (m_intItemCount + 1) * sizeof(DOC_ELEMENT));
	if (temp == NULL)				// If creation/expansion failed,
	{
		delete new_element.page;	// Delete page string.
		if (new_element.url)
			delete new_element.url;		// Delete URL string.
		if (new_element.comment)
			delete new_element.comment;
		return FALSE;				// Report failure to caller.
	}
	else							// Otherwise,
		m_pDocElements = temp;		// save new (possibly changed!) array pointer.
	// Add new element to array.
	m_pDocElements[m_intItemCount] = new_element;

	if (offset)						// If requested,
		*offset = m_intItemCount;	// return index of new element to caller.
	++m_intItemCount;				// Update item count.
	return (m_bItemsAdded = TRUE);	// Report success.
}

// Adds a path to the path list, or returns offset of
// existing path string if specified path is already there.
//
// Parameters:
//	psz_Path	= pointer to ASCIIZ string containing the path.
//	offset		= pointer to UINT for returned offset.
//
// Return Value: TRUE if successful, otherwise FALSE.
BOOL CSUDoc::Add_Path(LPCTSTR psz_Path, BOOL locked, UINT * pn_Offset)
{
	UINT i;
	PATH_ELEMENT* temp = NULL;
	CString * new_string = NULL;

	// String pointer must not be NULL!
	ASSERT(psz_Path != NULL);

	new_string = new CString(psz_Path);				// Create new CString containing path.
	if (new_string == NULL)							// If creation failed,
		return FALSE;								// report failure to caller.
	if (m_intPathCount)								// If path array is not empty,
	{
		for (i = 0; i < m_intPathCount; i++)		// See if specified path is already in the array.
		{
			if (m_pPathList[i].path->CompareNoCase(psz_Path) == 0)	// If it is,
			{
				delete new_string;	// Delete the new CString
				if (locked)
				{
					if (!m_pPathList[i].locked)
					{
						m_bItemsChanged = m_pPathList[i].locked = TRUE;
					}
				}
				if (pn_Offset)		// If requested,
					*pn_Offset = i;	// Give caller the offset to the existing string.
				return TRUE;		// Report success.
			}
		}
		// If path does not already exist, try to expand the array by one element.
		temp = (PATH_ELEMENT *) realloc(m_pPathList, (m_intPathCount + 1) * sizeof(PATH_ELEMENT));
	}
	else
		// Otherwise, create a new array.
		temp = (PATH_ELEMENT *) malloc(sizeof(PATH_ELEMENT));
	if (temp == NULL)			// If unable to create or expand the array,
	{
		delete new_string;		// Delete the new string.
		return FALSE;			// Report failure.
	}
	m_pPathList = temp;							// Save new (possibly changed!) array pointer.
	m_pPathList[m_intPathCount].path = new_string;	// Store pointer to new string.
	m_pPathList[m_intPathCount].processed = FALSE;
	m_pPathList[m_intPathCount].locked = locked;
	m_pPathList[m_intPathCount].selected = 2;
	if (new_string->Find('\\') == new_string->GetLength()-1)
		m_pPathList[m_intPathCount].parent = 0;
	else
	{
		CString s1, s2;
		int pos;

		s1 = new_string->Left(new_string->GetLength()-1);
		pos = s1.ReverseFind('\\') + 1;
		s2 = new_string->Left(pos);
		for (i = 1; i < m_intPathCount; i++)
		{
			if (s2.CompareNoCase(*m_pPathList[i].path) == 0)
			{
				m_pPathList[m_intPathCount].parent = i;
				break;
			}
		}
	}
	if (pn_Offset)								// If requested,
		*pn_Offset = m_intPathCount;			// give offset of new element to caller.
	++m_intPathCount;							// Update array count.
	m_bItemsAdded = (m_intPathCount > 1);
	return TRUE;								// Report success to caller.
}

// Imports MSIE bookmarks.
// Called recursively.
// On first call, "path" parm must be NULL indicating that
// we're processing the root "Favorites" folder.
// On subsequent calls, "path" parm is the relative path
// from the "Favorites" folder to the directory to be scanned.
// Returns FALSE on fatal error, otherwise TRUE.
BOOL CSUDoc::Scan_Folder(LPCTSTR path)
{
	IUniformResourceLocator* pIURL = NULL;
	IPersistFile* pIPF = NULL;
	UINT path_offset;
	CString cur_path, full_path, search_spec, full_spec, page_name, temp;
	LPSTR url = NULL;
	WORD szWide[_MAX_PATH];
	WIN32_FIND_DATA fd;
	HANDLE hFind;
	BOOL result;
	HRESULT hResult;

	if (path)		// If not processing the root folder,
	{
		cur_path = path;		// Set current path.
		cur_path += '\\';		// Append trailing backslash.
		if (!Add_Path(cur_path, TRUE, &path_offset))	// Append path to path list.
		{
			Report_Fatal_Error("Failed to create internal path object");
			return FALSE;
		}
		full_path = m_strIEPath + cur_path;	//Get fully qualified path to the folder.
	}
	else			// Otherwise,
	{
		cur_path.Empty();			// Current path is empty.
		path_offset = 0;			// It's the root folder
		full_path = m_strIEPath;	// And full path is the path to the "Favorites" folder itself.
	}
	search_spec = full_path + "*.*";				// Search for all files and folders.
	fd.dwFileAttributes = FILE_ATTRIBUTE_NORMAL;	// Any file attributes.
	hFind = FindFirstFile(search_spec, &fd);		// Begin the search.
	if (hFind != INVALID_HANDLE_VALUE)
	{
		while(TRUE)
		{
			if (fd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)		// Is current object a folder?
			{
				if ((strcmp(fd.cFileName, ".") != 0) &&				// and it's not "." or ".."?
					(strcmp(fd.cFileName, "..") != 0))
				{
					CString new_path = cur_path + fd.cFileName;		// Build path to new folder
					if (!Scan_Folder(new_path))						// and process it.
						return FALSE;
				}
			}
			else
			{
				page_name = fd.cFileName;							// Extract filename.
				temp = page_name.Right(4);							// Check extension.
				if (temp.CompareNoCase(".url") == 0)				// If it's a bookmark,
				{
					// Instantiate an IUniformResourceLocator object.
					hResult = CoCreateInstance(CLSID_InternetShortcut,
						NULL, CLSCTX_INPROC_SERVER, IID_IUniformResourceLocator, (void **) &pIURL);
					if (SUCCEEDED(hResult))
					{
						// Get pointer to the object's IPersistFile interface.
						hResult = pIURL->QueryInterface(IID_IPersistFile, (void **) &pIPF);
						if (!SUCCEEDED(hResult))
							pIURL->Release();
					}
					if (pIPF == NULL || pIURL == NULL)
					{
						Report_Fatal_Error("CSUDoc::Scan_Folder() could not obtain OLE interface pointer.");
						m_bErrorsLogged = TRUE;
						return FALSE;
					}
					full_spec = full_path + page_name;				// Get fully qualified path.
					MultiByteToWideChar(CP_ACP, 0, full_spec, -1, szWide, _MAX_PATH);	// Convert to OLE string.
					if (pIPF->Load(szWide, STGM_READ) == S_OK)		// Try to load the .URL object.
					{
						if (pIURL->GetURL(&url) == S_OK)			// If successful, retrieve the URL
						{
							if ((page_name.GetLength()) - 4 <= 50)	// Truncate the bookmark name at 50 characters.
								temp = page_name.Left(page_name.GetLength() - 4);
							else
								temp = page_name.Left(50);
							temp.TrimRight();
							result = Add_Item(path_offset, temp, url);	// Add bookmark to the document.
							if (!result)
								Report_Fatal_Error("Could not create internal bookmark item");
							pMalloc->Free(url);
						}
						else
						{
							temp.Format("Could not extract URL from %s\n", full_spec);
							m_pErrFile->WriteString(temp);
							m_bErrorsLogged = TRUE;
						}
						pIPF->SaveCompleted(szWide);	// Make sure the file is released.
						if (!result)					// Bail out of we failed to add the item to the document.
						{
							pIPF->Release();
							pIURL->Release();
							return FALSE;
						}
					}
					else
					{
						temp.Format("Failed to access file %s\n", full_spec);
						m_pErrFile->WriteString(temp);
						m_bErrorsLogged = TRUE;
					}
					pIPF->Release();
					pIURL->Release();
					pIPF = NULL;
					pIURL = NULL;
				}
			}
			if (!FindNextFile(hFind, &fd))
				break;
		}
		return TRUE;
	}
	Report_Fatal_Error("API function FindFirstFile failed.");
	return FALSE;
}


// Function determines whether or not the specified path
// has any unprocessed children. If so, returns TRUE
// and sets the UINT pointed to by the child parameter
// to the offset of the first child found.
BOOL CSUDoc::Has_Children(UINT offset, UINT * child)
{
	ASSERT(child != NULL);

	for (UINT i = 1; i < m_intPathCount; i++)			// For each folder accept the root,
	{
		if (!m_pPathList[i].processed)					// If path has not been passed,
		{
			if (m_pPathList[i].parent == offset)		// And it's parent is the target folder,
			{
				*child = i;								// Return offset to caller.
				return TRUE;							// Signal success.
			}
		}
	}
	return FALSE;		// Signal no unprocessed children found.
}

// Dumps data in text form. This is mainly for debugging purposes,
// although some folks may find this format useful.
void CSUDoc::Export_Text()
{
	CStdioFile file;
	CString line, strFileName;
	UINT i, j;
	BOOL found;

	CFileDialog dlg(FALSE, ".txt", "", OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT,
		"Text Files (*.txt)|*.txt||", NULL);
	dlg.m_ofn.lpstrTitle = "Export Text File";
	if (dlg.DoModal() != IDOK)
		return;
	strFileName = dlg.GetPathName();
	if (!file.Open(strFileName, CFile::modeCreate | CFile::modeWrite, NULL))
	{
		line.Format("Could not create or replace\n%s", strFileName);
		AfxMessageBox(line, MB_OK);
		return;
	}
	for (i=0; i < m_intPathCount; i++)
	{
		if (m_pPathList[i].selected == 1)
			continue;
		if (i==0)
			line = "Folder: Root";
		else
			line.Format("Folder: %s\n", *m_pPathList[i].path);
		file.WriteString(line);
		found = FALSE;
		for (j = 0; j < m_intItemCount; j++)
		{
			if (m_pDocElements[j].path_offset == i)
			{
				found = TRUE;
				line.Format("  Title: %s\n", *m_pDocElements[j].page);
				file.WriteString(line);
				line.Format("    URL: %s\n", *m_pDocElements[j].url);
				file.WriteString(line);
			}
		}
		if (!found)
			file.WriteString("  Folder contains no bookmarks.\n");
	}
	file.Close();
}


// Extract bookmarks from Netscape's BOOKMARK.HTM file.
UINT CSUDoc::Parse_NS_File()
{
	CStdioFile file;
	CString line1, line2;
	int pos;
	UINT result = 0;

	// First, make sure we can get access to the file.
	if (!file.Open(m_strNSPath, CFile::modeRead | CFile::typeText | CFile::shareDenyWrite, NULL))
	{
		line1.Format("Could not access %s", m_strNSPath);
		Report_Fatal_Error(line1);
		return IDP_IMPORT_FAIL;
	}
	m_pFile = &file;
	// Extract the user's root folder title.
	while (TRUE)
	{
		if (!file.ReadString(line1))
		{
			Report_Fatal_Error("Unexpected end of file");
			result = IDP_IMPORT_FAIL;
			goto ErrorExit;
		}
		if ((pos = line1.Find("<H1>")) >= 0)
		{
			pos += 4;
			line2 = line1.Mid(pos);
			if ((pos = line2.Find("</H1>")) == -1)
			{
				line2.Format("HTML error in:\n%s\nMissing </H1> tag. Processing aborted", line1);
				Report_Fatal_Error(line2);
				result = IDP_IMPORT_FAIL;
				goto ErrorExit;
			}
			m_strNSPageTitle = line2.Left(pos);
			break;
		}
	}
	// Now recurse our way through the file, reporting error if anything barfs.
	if ((result = Parse_NS_Folder()))
		goto ErrorExit;
	file.Close();
	m_pFile = NULL;
	return 0;
ErrorExit:
	file.Close();
	m_pFile = NULL;
	return result;
}

// This recursive routine parses the folders in Netscape's BOOKMARK.HTM file.
// The first time it is called, both parameters are NULL, indicating that
// we're starting with the root folder.
BOOL CSUDoc::Parse_NS_Folder(LPCTSTR old_path, LPCTSTR new_path)
{
	CString cur_path, line1, line2, debug;
	int pos, pos1;
	static UINT blank_folder_count = 0;
	UINT path_offset, result;
	LONG add_date, mod_date, visit_date;
	CString page, url, comment, err_msg;
	BOOL in_comment = FALSE, got_url = FALSE;
	LPSTR new_url = NULL;
	CBadDlg* dlg = NULL;
	HRESULT hResult;


	if (old_path == NULL)	// Root folder?
	{
		cur_path.Empty();		// Path is blank.
	}
	else	//Otherwise,
	{
		// Check path length.
		if (strlen(old_path) + strlen(new_path) + m_intBasePathLen > 200)
		{
			// If it's too long, try to create our default "Relocated" path.
			if (!Add_Path("Relocated\\", FALSE))
			{
				Report_Fatal_Error("Could not create Relocated folder");
				return IDP_IMPORT_FAIL;
			}
			// If that worked, reset the parent of the current folder
			// to our "Relocated" folder, then make sure we can create a path for the folder itself.
			cur_path = "Relocated\\";
			cur_path += new_path;
			if (!Add_Path(cur_path, FALSE))
			{
				err_msg.Format("Could not create %s folder", cur_path);
				Report_Fatal_Error(err_msg);
				return IDP_IMPORT_FAIL;
			}
			// Let the user know that we had to move the folder.
			err_msg.Format("Folder \"%s\"\n"
				"could not be created because the path is too long\n"
				"Folder was moved to a root folder named \"Relocated\"", new_path);
			m_pErrFile->WriteString(err_msg);
			m_bErrorsLogged = TRUE;
		}
		else	// Path is not too long,
		{
			// So just tack the new folder name onto the existing one.
			cur_path = old_path;
			cur_path += new_path;
		}
	}
	// Search for <DL><p> tag, indicating the start of a folder's contents.
	while (TRUE)
	{
		if (!m_pFile->ReadString(line1))	// Report failure if EOF reached.
		{
			Report_Fatal_Error("Unexpected end of file");
			return IDP_IMPORT_FAIL;
		}
		if (line1.Find("<DL><p>") != -1)
			break;
	}
	if (cur_path.IsEmpty())			// If path string is empty,
	{
		path_offset = 0;			// We're processing the root folder.
	}
	else
	{
		// Otherwise, either add or locate the current folder's path
		// and retrieve its offset.
		if (!Add_Path(cur_path, FALSE, &path_offset))
		{
			Report_Fatal_Error("Failed to create internal folder item");
			return IDP_IMPORT_FAIL;
		}
	}
	while(TRUE)
	{
		if (!m_pFile->ReadString(line1))			// Error if EOF reached before end of folder tag is found.
		{
			Report_Fatal_Error("Unexpected end of file");
			return IDP_IMPORT_FAIL;
		}
		debug = line1;
		if (debug[debug.GetLength()-1] == '\r')
		{
			line2 = debug.Left(debug.GetLength() -1);
			debug = line2;
		}
		//Folder tags are of the form "<DT><H3 misc data>Folder Name</H3>"
		if ((pos = line1.Find("<DT><H3")) != -1)	// Branch if folder tag.
		{
//			goto ParseFolder;
			// If there's a Bookmark pending, write it to the file. This seems clumsy, but it's the
			// easiest way to handle multi-line comments.
			if (got_url)
			{
				got_url = FALSE;			// Reset pending URL flag.
				in_comment = FALSE;			// Reset comment processing flag.
				if (comment.IsEmpty())		// If no comment, pass NULL, otherwise pass the comment itself.
					result = Add_Item(path_offset, page, url, NULL, add_date, mod_date, visit_date);
				else
					result = Add_Item(path_offset, page, url, comment, add_date, mod_date, visit_date);
				if (!result)
				{
					Report_Fatal_Error("Failed to add item to document");
					return IDP_IMPORT_FAIL;
				}
			}
			pos += 7;								// Skip past "<DT><H3"
			line2 = line1.Mid(pos);					// Truncate the line.
			if ((pos = line2.Find('>')) == -1)		// Make sure end of <H3 tag exists.
			{
				err_msg.Format("HTML error in line:\n%s\nMissing '>' marker", debug);
				Report_Fatal_Error(err_msg);
				return IDP_IMPORT_FAIL;
			}
			pos += 1;								// Move past end of <H3 tag.
			line1 = line2.Mid(pos);
			if ((pos = line1.Find("</H3>")) == -1)	// Make sure </H3> end marker is present.
			{
				err_msg.Format("HTML error in line:\n%s\nMissing </H3> tag", debug);
				Report_Fatal_Error(err_msg);
				return IDP_IMPORT_FAIL;
			}
			line2 = line1.Left(pos);				// Grab the folder name.
			if (line2.IsEmpty())					// If name is blank,
			{
				++blank_folder_count;				// Provide default name.
				line2.Format("Folder (%u)", blank_folder_count);
				if (cur_path.IsEmpty())				// Format and write appropriate
					line1 = "Root Folder";			// message to the error log.
				else
				{
					line1 = "Folder ";
					line1 += cur_path.Left(cur_path.GetLength() -1);
				}
				err_msg.Format("%s contains a folder with no name\nDefault name %s provided\n", line1, line2);
				m_pErrFile->WriteString(err_msg);
			}
			Fix_FileName(line2);					// Clean up illegal characters for MSIE compatibility.
			line2 += '\\';
			result = Parse_NS_Folder(cur_path, line2);	// Try to parse new folder.
			if (result)
				return result;		// Report error if necessary.
			continue;
		}
		// Bookmark tags are of the form "<DT><A HREF="url text" misc data>Page Name</A>"
		if ((pos = line1.Find("<DT><A")) != -1)		// Branch if URL tag.
		{
//			goto ParseURL;
			// If there's a Bookmark pending, write it to the file. This seems clumsy, but it's the
			// easiest way to handle multi-line comments.
			if (got_url)
			{
				got_url = FALSE;			// Reset pending URL flag.
				in_comment = FALSE;			// Reset comment processing flag.
				if (comment.IsEmpty())		// If no comment, pass NULL, otherwise pass the comment itself.
					result = Add_Item(path_offset, page, url, NULL, add_date, mod_date, visit_date);
				else
					result = Add_Item(path_offset, page, url, comment, add_date, mod_date, visit_date);
				if (!result)
				{
					Report_Fatal_Error("Failed to add item to document");
					return IDP_IMPORT_FAIL;
				}
			}
			page.Empty();			// Reset strings for the bookmark.
			url.Empty();
			comment.Empty();
			pos += 6;				// Skip past the "<DT><A" marker.
			line2 = line1.Mid(pos);
			if ((pos = line2.Find("ADD_DATE=\"")) == -1)			// ADD_DATE tag present?
				add_date = 0;										// No date if so.
			else													// Otherwise,
			{
				pos += 10;											// Strip off text before the value.
				line1 = line2.Mid(pos);
				add_date = strtoul(line1, NULL, 10);				// And convert ASCII to unsigned long.
			}
			// Do the same for the "LAST_MODIFIED" and "LAST_VISIT" tags.
			if ((pos = line2.Find("LAST_MODIFIED=\"")) == -1)
				mod_date = 0;
			else
			{
				pos += 15;
				line1 = line2.Mid(pos);
				mod_date = strtoul(line1, NULL, 10);
			}
			if ((pos = line2.Find("LAST_VISIT=\"")) == -1)
				visit_date = 0;
			else
			{
				pos += 12;
				line1 = line2.Mid(pos);
				visit_date = strtoul(line1, NULL, 10);
			}
			if ((pos = line2.Find("HREF=\"")) == -1)		// Locate start of URL text.
			{
				Report_HTML_Err(debug, "Missing HREF= tag");
				continue;
			}
			pos += 6;										// Skip past HREF="
			line1 = line2.Mid(pos);
			if ((pos = line1.Find('\"')) == -1)				// Search for trailing quote.
			{
				Report_HTML_Err(debug, "Missing close quote");
				continue;
			}
			url = line1.Left(pos);							// Extract URL text.
			if (url.IsEmpty())								// Error if URL is blank.
			{
				Report_HTML_Err(debug, "URL string is empty");
				continue;
			}
			if ((pos = line1.Find('>')) == -1)				// Search for close of <A tag.
			{
				Report_HTML_Err(debug, "Missing > marker");
				continue;
			}
			pos += 1;										// Skip past the ">"
			line2 = line1.Mid(pos);							// Extract page title.
			if ((pos = line2.Find("</A>")) == -1)			// Search for "</A>" end marker.
			{
				Report_HTML_Err(debug, "Missing </A> tag");
				continue;
			}
			if (pos <= 50)									// Truncate page title to 50 chars if necessary
				page = line2.Left(pos);						// for compatibility with MSIE.
			else
				page = line2.Left(50);
			if (page.IsEmpty())								// If page title is blank,
			{
				line2 = url;								// Create a title from the URL
				page = line2.Left(50);
				Fix_FileName(page);								// Convert illegal characters (MSIE compat. again!)
				err_msg.Format("Bookmark for URL \"%s\" has no name\nDefault name \"%s\" provided\n", url, page);
				m_pErrFile->WriteString(err_msg);
				m_bErrorsLogged = TRUE;
			}
			else
				Fix_FileName(page);
			// Now, check the URL syntax.
			while (TRUE)
			{
				hResult = TranslateURL(url, TRANSLATEURL_FL_GUESS_PROTOCOL, &new_url);
				switch (hResult)
				{
				case S_OK:
					url = new_url;						// Save it.
					LocalFree(new_url);
					new_url = NULL;
					got_url = TRUE;
					break;
				case S_FALSE:
					if (url.Find("://") > 0)
					{
						got_url = TRUE;
					}
					else
					{
						if (url.Find("mailto:") == 0 || url.Find("MAILTO:") == 0)
							got_url = TRUE;
						else
							got_url = FALSE;
					}
					break;
				default:
					m_pErrFile->WriteString("Parse_NS_Folder(): TranslateURL() failed\n");
					m_bErrorsLogged = TRUE;
					continue;
				}
				// If not able to resolve syntax, let user decide.
				if (got_url)
					break;
				else
				{
					if (dlg == NULL)				// If dialog object doesn't exist,
						dlg = new CBadDlg;			// Create it.
					pos = url.Find('@');
					pos1 = url.Find('.');
					if ((pos >= 0) && (pos1 >= 0) && (pos1 > pos))
					{
						line2 = "mailto:";			// Tack on the "mailto:" prefix.
						line2 += url;
						dlg->m_txtNewURL = line2;	// Present that as our "guess".
					}
					else
						dlg->m_txtNewURL = url;		// Otherwise, use the URL verbatim.
					dlg->m_lblBadPageName = page;	// Prime dialog with page name
					dlg->m_txtOriginalURL = url;	// and original URL
					dlg->m_bSkip = FALSE;			// Make sure the "skip" flag is reset.
					dlg->DoModal();					// Launch the dialog. If user said "OK",
					if (dlg->m_bSkip)			// And the "skip" flag is set,
					{
						got_url = FALSE;		// Ignore this bookmark
						break;					// and continue.
					}
					else
					{
						url = dlg->m_txtNewURL;		// Otherwise, retrieve URL from the dialog and try again.
						got_url = TRUE;
					}
				}
			}
			if (dlg)			// If dialog object exists,
			{
				delete dlg;		// delete it
				dlg = NULL;		// and NULL its pointer.
			}
			continue;										// Continue parsing.
		}
		// Comment tags are of the form "<DD> comment text"
		// Note that comments may span multiple lines, and the end of the comment
		// does _not_ have a corresponding </DD> marker -- the comment ends when
		// the next Folder, BookMark, or End of Folder tag is reached.
		if ((pos = line1.Find("<DD>")) != -1)		// Branch if comment tag.
		{
//			goto ParseComment;
			in_comment = TRUE;			// Set flag to indicate that we're processing a comment sequence.
			pos += 4;					// Skip past the <DD> marker.
			comment = line1.Mid(pos);	// Extract remainder of line verbatim.
			continue;					// Continue parsing.
		}
		if ((pos = line1.Find("</DL><p>")) != -1)	// Done processing when end of folder tag is reached.
			break;
		if (in_comment)								// If none of the above, and flag is set, we're in
		{											// the middle of a comment sequence.
			comment += '\n';						// Append CR/LF to existing comment.
			comment += line1;						// Tack on comment text verbatim.
		}
//		continue;
//ParseFolder:
//ParseURL:
//ParseComment:
	}
	// If there's a URL pending, write it to the file.
	if (got_url)
	{
		got_url = FALSE;			// Reset pending URL flag.
		in_comment = FALSE;			// Reset comment processing flag.
		if (comment.IsEmpty())		// If no comment, pass NULL, otherwise pass the comment itself.
			result = Add_Item(path_offset, page, url, NULL, add_date, mod_date, visit_date);
		else
			result = Add_Item(path_offset, page, url, comment, add_date, mod_date, visit_date);
		if (!result)
		{
			Report_Fatal_Error("Failed to add item to document");
			return IDP_IMPORT_FAIL;
		}
	}
	// Signal success to caller.
	return 0;
}


// Function replaces any illegal characters in the
// filename passed with a '$' sign. This is required for compatibility
// with MSIE bookmarks and folders, which represent actual files and
// folders on disk.
void CSUDoc::Fix_FileName(CString& fname)
{
	int  len = fname.GetLength();
	char *buff1 = new char[len + 1];
	char *buff2 = new char[len + 1];
	char *p1 = buff1, *p2 = buff2;

	strcpy(buff1, fname);	// Copy name to local buffer.
	while (*p1)
	{
		if (strchr("\\/:?*\"<>|", *p1) != NULL)	// Illegal char for filenames?
			*p2 = '$';			// Change it to '$'
		else
		{
			if (*p1 == '\t')	// Tab?
				*p2 = ' ';		// Change to space.
			else				// Otherwise,
				*p2 = *p1;		// transfer char verbatim.
		}
		++p1;
		++p2;
	}
	*p2 = '\0';		// Make sure string is terminated.
	--p2;
	// Strip any trailing spaces.
	while (p2 >= buff2 && *p2 == ' ')
		*p2-- = '\0';
	// Install modified string.
	fname = buff2;
	fname.TrimLeft();
	delete [] buff1;
	delete [] buff2;
}

// Callback function used by qsort() library function to
// sort our list of URLs. This is a two-key sort,
// first key is the page name, second key is the URL's parent folder.
int __cdecl Compare_Items(const void * p1, const void * p2)
{
	ASSERT(p1 != NULL && p2 != NULL);

	const DOC_ELEMENT* i1 = (DOC_ELEMENT*) p1;
	const DOC_ELEMENT* i2 = (DOC_ELEMENT*) p2;
	int result;

	result = i1->page->CompareNoCase(*i2->page);	// Are strings identical?
	if (result)										// If not,
		return result;								// return result to caller.
	// If strings are identical, subkey is the directory in which they reside.
	if (i1->path_offset	> i2->path_offset)
		return 1;
	if (i1->path_offset < i2->path_offset)
		return -1;
	else
		return 0;
}

// Resets the "processed" flags used by recursive functions.
void CSUDoc::Reset_Path_Flags()
{
	for (UINT i = 0; i < m_intPathCount; m_pPathList[i++].processed = FALSE);
}

// Function reads MSIE and/or Netscape bookmarks and
// adds them to the current document.
void CSUDoc::Do_Import()
{
	CString line1;
	BOOL fatal = FALSE;

	m_bItemsAdded = m_bItemsChanged = FALSE;
	if (!Start_Error_Log())
	{
		AfxMessageBox(IDP_ERR_LOG_FAIL);
		return;
	}
	Add_Path("\\", TRUE);		// Add path for root folder.
	m_pStatDlg = new CStatDlg;
	m_pStatDlg->CenterWindow();
	m_pStatDlg->ShowWindow(SW_SHOW);
	if (m_bNSPresent)
	{
		m_pStatDlg->Show_Message("Importing Netscape 3.x bookmarks");
		m_pErrFile->WriteString("Beginning Netscape 3.x import\n");
		if (Parse_NS_File() == IDP_IMPORT_FAIL)
			fatal = TRUE;
		if (!m_bErrorsLogged)
			m_pErrFile->WriteString("Netscape 3.x import complete ... no errors.\n\n");
	}
	if (m_bNSCPresent)
	{
		m_pStatDlg->Show_Message("Importing Netscape 4.x bookmarks");
		m_pErrFile->WriteString("Beginning Netscape 4.x import\n");
		line1 = m_strNSPath;
		m_strNSPath = m_strNSCPath;
		if (Parse_NS_File() == IDP_IMPORT_FAIL)
			fatal = TRUE;
		m_strNSPath = line1;
		if (!m_bErrorsLogged)
			m_pErrFile->WriteString("Netscape 4.x import complete ... no errors.\n\n");
	}
	if (m_bIEPresent)
	{
		m_pStatDlg->Show_Message("Importing MSIE bookmarks");
		m_pErrFile->WriteString("\nBeginning MSIE import\n\n");
		if (!Scan_Folder(NULL))
			fatal = TRUE;
		else
			m_pErrFile->WriteString("MSIE import complete ... no errors.\n\n");
	}
	Do_Import_Cleanup();
	m_pStatDlg->DestroyWindow();
	delete m_pStatDlg;
	m_pStatDlg = NULL;
	End_Error_Log();
	if (m_bErrorsLogged)
		Query_Show_Log(fatal);
	SetModifiedFlag(m_bItemsChanged || m_bItemsAdded);
}


// Creates MSIE bookmarks from the current document.
// This function is recursive, so it is called once
// with the path_offset parm set to 0, indicating to start with
// the root folder. It then calls itself with the offset of each
// child folder until all folders have been processed.
// Returns FALSE and stops recursion only if a folder cannot be created.

// IMPORTANT: Because the user's Favorites folder may be used by other
// apps besides MSIE and may contain items that are not .URL files,
// this function will not remove any existing folders from
// the user's "Favorites" folder tree, and it will not touch
// any existing "non-url" objects found along the way.

// Function works by recursing through the tree starting at the root.
// If a folder does not exist, it is created.
// If a folder does exist, any existing .URL files in that folder are removed, if possible
//  (function does not attempt to remove or alter existing read-only files).
// Process repeats until the end of a branch is reached and all folders
// along the way have been either created or cleaned. The individual bookmarks (.url files)
// within each folder are created as the function walks back up the tree.
// Failure to create a bookmark (e.g., read-only file, invalid URL syntax, etc.)
// presents a warning message but does not stop the process.

BOOL CSUDoc::Export_MSIE_Path(UINT path_offset)
{
	IUniformResourceLocator *pIURL = NULL;
	IPersistFile *pIPF = NULL;
	CString msg;
	CString path1, path2;
	CString filename;
	int result, errval;
	UINT i, child, selected;
	WORD szWide[_MAX_PATH];
	HRESULT hResult;

	// If path is not selected for export,
	selected = m_pPathList[path_offset].selected;
	m_pPathList[path_offset].processed = TRUE;
	path1 = m_strIEPath;
	if (path_offset != 0)
		path1 += *m_pPathList[path_offset].path;
	path2 = path1.Left(path1.GetLength()-1);
	if (selected != 1)								// If folder is selected,
	{
		msg.Format("Creating folder %s", path2);	// Try to create it.
		pStatDlg->Show_Message(msg);
		result = _mkdir(path2);
		errval = errno;
		if (result == -1)					// If creation failed
		{
			if (errval == EEXIST)			// because the folder already exists,
				Clean_Folder(path1);		// just clean it out.
			else
				return FALSE;				// Otherwise report failure and quit.
		}
		m_pPathList[path_offset].locked = TRUE;
	}
	else						// Folder is not selected,
	{
		Clean_Folder(path1);	// so just zap any .URL files it contains.
	}
	m_pPathList[path_offset].processed = TRUE;
	while(Has_Children(path_offset, &child))
	{
		if (!Export_MSIE_Path(child))
			return FALSE;
	}
	msg.Format("Processing folder %s", path2);		// Let user know what's going on.
	pStatDlg->Show_Message(msg);
	// If folder is selected,
	if (selected != 1)
	{
		for (i = 0; i < m_intItemCount; i++)
		{
			hResult = CoCreateInstance(CLSID_InternetShortcut,
				NULL, CLSCTX_INPROC_SERVER, IID_IUniformResourceLocator, (void **) &pIURL); 
			// If that worked, try to get pointer to IUniformResourceLocator's
			// IPersistFile interface.
			if (SUCCEEDED(hResult))
			{
				hResult = pIURL->QueryInterface(IID_IPersistFile, (void **) &pIPF);
				if (!SUCCEEDED(hResult))
					pIURL->Release();
			}
			if (pIURL == NULL || pIPF == NULL)
			{
				AfxMessageBox("OLE Init failed", MB_ICONINFORMATION);
				return FALSE;
			}
			if (m_pDocElements[i].path_offset != path_offset)
				continue;
			filename = path1;
			filename += *m_pDocElements[i].page;
			filename += ".url";
			MultiByteToWideChar(CP_ACP, 0, filename, -1, szWide, _MAX_PATH);
			if (pIURL->SetURL(*m_pDocElements[i].url, IURL_SETURL_FL_GUESS_PROTOCOL) == S_OK)
			{
				if (pIPF->Save(szWide, TRUE) == S_OK)
				{
					pIPF->SaveCompleted(szWide);
				}
				else
				{
					msg.Format("File creation failed.\n\nFilename: %s", filename);
					AfxMessageBox(msg, MB_ICONINFORMATION);
				}
			}
			else
			{
				msg.Format("Invalid URL syntax.\n\nPage: %s\nURL:%s", *m_pDocElements[i].page, *m_pDocElements[i].url);
				AfxMessageBox(msg, MB_ICONINFORMATION);
			}
			pIPF->Release();
			pIURL->Release();
			pIPF = NULL;
			pIURL = NULL;
		}
	}
	// If not selected,
	else
	{
		if (path_offset != 0)								// If folder is not the root,
		{
			if (_rmdir(path2) == 0)							// try to remove it and
				m_pPathList[path_offset].locked = FALSE;	// mark it as unlocked if successful.
		}
		else												// Folder is the root,
			m_pPathList[path_offset].locked = TRUE;			// so lock it.
	}
	return TRUE;
}

// Called by Export_MSIE_Folder.
// Prepares the specified directory by deleting all .URL files it contains.
void CSUDoc::Clean_Folder(LPCTSTR path)
{
	CString search_spec, file_spec;
	WIN32_FIND_DATA fd;
	HANDLE handle;

	search_spec = path;
	search_spec += "*.url";
	if ((handle = FindFirstFile(search_spec, &fd)) != INVALID_HANDLE_VALUE)
	{
		do
		{
			if ((fd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) == 0)
			{
				file_spec = path;
				file_spec += fd.cFileName;
				try {
					CFile::Remove(file_spec);
				}
				catch (CFileException* e)
				{
					e->Delete();
				}
			}
		} while (FindNextFile(handle, &fd));
		FindClose(handle);
	}
}

// Called by ID_EXPORT commands.
// Prepares for, then executes the MSIE export function.
void CSUDoc::Do_MSIE_Export()
{
	ASSERT(pStatDlg == NULL);		// Status dialog pointer should already be NULL.
	pStatDlg = new CStatDlg;		// Create new Status dialog object.
	BOOL result;
	ASSERT(pStatDlg != NULL);		// Make sure creation succeeded.
	pStatDlg->CenterWindow();		// Center the dialog.
//	Reset_Path_Flags();				// Prepare "processed" flags for recursion.
	for (UINT i = 0; i < m_intPathCount; i++)
	{
		m_pPathList[i].processed = m_pPathList[i].locked = FALSE;
	}
	result = Export_MSIE_Path(0);	// Do it!
	pStatDlg->DestroyWindow();		// Close the dialog.
	delete pStatDlg;				// Delete the dialog object.
	pStatDlg = NULL;				// NULL object pointer.
	UpdateAllViews(NULL);
	if (!result)					// Indicate final status to user.
		AfxMessageBox("MSIE Export failed.", MB_ICONEXCLAMATION);
}


// Deletes the specified item from the document.
void CSUDoc::Delete_Item(UINT offset)
{
	DOC_ELEMENT* new_array;
	void *p1, *p2;
	size_t size = sizeof(DOC_ELEMENT);

	ASSERT(offset < m_intItemCount);

	// Delete CStrings belonging to the item, if they exist.
	if (m_pDocElements[offset].page)
		delete m_pDocElements[offset].page;
	if (m_pDocElements[offset].url)
		delete m_pDocElements[offset].url;
	if (m_pDocElements[offset].comment)
		delete m_pDocElements[offset].comment;
	// Adjust the item count.
	--m_intItemCount;
	// Mark document as dirty.
	SetModifiedFlag();
	if (m_intItemCount == 0)		// No items left?
	{
		delete [] m_pDocElements;	// Delete the entire array,
		m_pDocElements = NULL;		// NULL the pointer,
		return;						// and we're done.
	}
	// If item being deleted is not the last one in the array,
	if (offset < m_intItemCount)
	{
		// Close the gap left by the deleted item
		p1 = &m_pDocElements[offset];
		p2 = &m_pDocElements[offset + 1];
		memmove(p1, p2, (m_intItemCount - offset) * size);
	}
	// Reallocate the array.
	new_array = (DOC_ELEMENT*) realloc(m_pDocElements, m_intItemCount * size);
	ASSERT(new_array != NULL);
	m_pDocElements = new_array;
}

// Function takes a path string (with trailing backslash)
// and replaces it with the name of the last folder in the path.
void CSUDoc::Friendly_Name(CString & path)
{
	CString s1;
	int pos;

	if (path.GetLength() == 1)
	{
		path = "Root Folder";
		return;
	}
	s1 = path.Left(path.GetLength() - 1);
	pos = s1.ReverseFind('\\') + 1;
	if (pos)
		path = s1.Mid(pos);
	else
		path = s1;
}

BOOL CSUDoc::Move_Folder(UINT source, UINT dest)
{
	CString strSource = *m_pPathList[source].path;
	CString strDest = *m_pPathList[dest].path;
	CString scratch, s1, s2;
	int base_len, source_len, targ_len;
	UINT i, cur, parent, old_parent;
	BOOL all, none;

	old_parent = m_pPathList[source].parent;		// Get parent of folder that moved.
	base_len = m_strIEPath.GetLength();
	if (dest != 0)									// If destination is not the root,
		base_len += strDest.GetLength();			// Adjust length of destination path.
	if (old_parent == 0)							// Get length of parent path for source folder.
		source_len = 0;
	else
		source_len = m_pPathList[m_pPathList[source].parent].path->GetLength();
	for (i = 1; i < m_intPathCount; i++)			// Loop through the paths
	{
		if (m_pPathList[i].path->Find(strSource) == 0)					// If it will be moved,
		{
			targ_len = m_pPathList[i].path->GetLength() - source_len;	// Make sure resulting path
			if ((targ_len + base_len) > 200)							// won't be too long.
				return FALSE;											// Can't move if it is.
		}
	}
	s1 = strSource;								// Get source path.
	Friendly_Name(s1);							// Extract last folder name.
	for (i=1; i < m_intPathCount; i++)			// Walk through the paths again.
	{
		if (m_pPathList[i].parent == dest)		// Is path a child of the destination?
		{
			s2 = *m_pPathList[i].path;			// Grab the path.
			Friendly_Name(s2);					// Extract last dir in the path.
			if (s1.CompareNoCase(s2) == 0)		// If name is the same,
				return FALSE;					// can't move it.
		}
	}
	for (i=1; i < m_intPathCount; i++)			// Loop through the paths yet again.
	{
		if (m_pPathList[i].path->Find(strSource) == 0)		// Does it belong to the source folder?
		{
			if (dest != 0)									// Build new name if so.
				scratch = strDest + m_pPathList[i].path->Mid(source_len);
			else
				scratch = m_pPathList[i].path->Mid(source_len);
			*m_pPathList[i].path = scratch;					// Install the new name.
			if (i == source)								// If it's the source path,
				m_pPathList[i].parent = dest;				// set its new parent offset.
		}
	}
	// Update the selection state.
	Reset_Path_Flags();
	// Handle source folder first (folder can't be moved to its own children).
	if (Has_Children(old_parent, &cur))
	{
		all = TRUE;									// Flag indicates all subfolders are selected.
		none = TRUE;								// Flag indicates no subfolders are selected.
		for (i = 1; i < m_intPathCount; i++)		// Walk the path array.
		{	
			if (m_pPathList[i].parent != old_parent)	// Skip if not a child of the source folder.
				continue;
			if (m_pPathList[i].selected != 2)			// If not fully selected,
				all = FALSE;							// then not all children are selected.
			if (m_pPathList[i].selected != 1)			// If fully or partially selected,
				none = FALSE;							// then at least some of the children are selected.
		}
		if (none)										// No children selected at all?
			m_pPathList[old_parent].selected = 1;		// Then it's not selected either.
		else
		{
			if (all)									// All children fully selected?
				m_pPathList[old_parent].selected = 2;	// Then so is parent.
			else
				m_pPathList[old_parent].selected = 3;	// Otherwise, parent is partially selected.
		}
	}
	else												// Old source folder has no remaining children,
	{
		if (m_pPathList[old_parent].selected == 3)		// so if it was partially selected,
			m_pPathList[old_parent].selected = 2;		// it is now fully selected.
		// (If it wasn't selected before the move, it remains unselected!)
	}
	// Now handle the destination folder.
	cur = source;
	do													// Loop works from source folder upwards.
	{
		parent = m_pPathList[cur].parent;				// Get parent of current folder.
		all = TRUE;										// Reset flags.
		none = TRUE;
		for (i = 1; i < m_intPathCount; i++)			// Same algorithm as previous loop.
		{
			if (m_pPathList[i].parent != parent)
				continue;
			if (m_pPathList[i].selected != 2)
				all = FALSE;
			if (m_pPathList[i].selected != 1)
				none = FALSE;
		}
		// Slight difference here; since we're working from the bottom up,
		// we know that the target folder has children.
		if (none)
			m_pPathList[parent].selected = 1;
		else
		{
			if (all)
				m_pPathList[parent].selected = 2;
			else
				m_pPathList[parent].selected = 3;
		}
		cur = parent;	// Move up a level.
	} while (cur);		// Done once we reach the root.
	return TRUE;		// Success!
}

// Function resolves duplicate bookmark names
// Specifically, for bookmarks in the same folder with the same name but
// different URLs). The import process automatically discards bookmarks
// that are true duplicates, with same name _and_ URL.
// Returns TRUE if any bookmarks were renamed.
BOOL CSUDoc::Resolve_Dupes()
{
	BOOL found = FALSE, result = FALSE, valid;
	DUP_ELEMENT* pDupes;
	CString* scratch;
	CString s1, s2, s3, msg;
	int len1, len2;
	UINT i, j, k, l, count, dup_count, anchor;

	m_pErrFile->WriteString("\nResolving duplicate bookmarks:\n\n");
	// Make array same size as the document.
	pDupes = new DUP_ELEMENT[m_intItemCount];
	ASSERT(pDupes!= NULL);

	// Outer loop walks the folder array.
	for (i = 0; i < m_intPathCount; i++)
	{
		count = 0;								// Reset element count.
		if (i == 0)
			msg = "Processing Root Folder\n";
		else
			msg.Format("Processing %s\n", *m_pPathList[i].path);
		m_pErrFile->WriteString(msg);
		for (j = 0; j < m_intItemCount; j++)	// Walk the document.
		{
			if (m_pDocElements[j].path_offset != i)	// Bookmark in current folder?
				continue;
			scratch = new CString(*m_pDocElements[j].page);	// Copy its name.
			ASSERT (scratch != NULL);
			pDupes[count].name = scratch;		// and add it to the array.
			pDupes[count].offset = j;
			++count;
		}
		found = FALSE;
		if (count > 1)							// Nothing to do if folder has only one bookmark!
		{
			// Sort the names into alphabetical order.
			qsort((void*) pDupes, count, sizeof(DUP_ELEMENT), Compare_Dupes);
			// Walk through the array.
			for (j = 0; j < count -1; j++)
			{
				// Two elements with same name?
				if (pDupes[j].name->CompareNoCase(*pDupes[j + 1].name) == 0)
				{
					found = TRUE;		// Set flag.
					dup_count = 1;		// Initialize count for numeric suffixes.
					anchor = j;			// Save position of first element.
					// Walk through the remainder of the array.
					for (k = anchor + 1; k < count; ++dup_count)
					{
						s1 = *pDupes[k].name;			// Copy folder name.
						len1 = s1.GetLength();			// Get its length.
						s2.Format(" (%u)", dup_count);	// Create numeric suffix.
						len2 = s2.GetLength();			// Get its length.
						if ((len1 + len2) < 50)			// If it's not too long,
							s1 += s2;					// just append the suffix.
						else
						{
							s3 = s1.Left(50 - len2);	// Otherwise chop the name
							s1 = s3 + s2;				// then append the suffix.
						}
						valid = TRUE;					// Set flag.
						for (l = 0; l < count; l++)		// Make sure new name doesn't already exist!
						{
							if (s1.CompareNoCase(*pDupes[l].name) == 0)
							{
								valid = FALSE;
								break;
							}
						}
						if (!valid)						// If new name already exists,
							continue;					// Try again with next suffix number.
						// Write message to the log.
						msg.Format("  Duplicate  \"%s\"\n"
								   "  renamed to \"%s\"\n\n", *pDupes[k].name, s1);
						m_pErrFile->WriteString(msg);
						m_bErrorsLogged = TRUE;
						// Install the new name.
						*pDupes[k].name = s1;
						// Next element
						++k;
						// Done if name doesn't match anchor name.
						if (k < count)
							if (pDupes[anchor].name->CompareNoCase(*pDupes[k].name) != 0)
								break;
					}
					j = k - 1;		// Ajust array position and keep going.
				}
			}
		}
		if (found)					// If any dupes were found,
		{
			result = TRUE;			// Set flag.
			for (j = 0; j < count; j++)	// Then copy the new names back into the document.
				*m_pDocElements[pDupes[j].offset].page = *pDupes[j].name;
		}
		for (j = 0; j < count; j++)	// Clean out work array.
			delete pDupes[j].name;
	}	// Continue with next folder.
	delete [] pDupes;		// Delete the work array.
	// Write string to log if no dupes were found.
	if (!result)
		m_pErrFile->WriteString("\nNo duplicates found.\n");
	else
		m_bItemsChanged = TRUE;
	return result;
}

// Callback function used by qsort() in Resolve_Dupes()
int __cdecl Compare_Dupes(const void * p1, const void * p2)
{
	ASSERT(p1 != NULL && p2 != NULL);

	const DUP_ELEMENT* i1 = (DUP_ELEMENT*) p1;
	const DUP_ELEMENT* i2 = (DUP_ELEMENT*) p2;

	return (i1->name->CompareNoCase(*i2->name));

}

// Called by CSUView when a folder is renamed.
// Input: offset = Offset to the path whose name is changing
//        new_name = New "Friendly" folder name (without trailing backslash)
// Returns FALSE if new name would cause any affected paths
// to be too long.
// Otherwise, adjusts all affected path strings and returns TRUE.
BOOL CSUDoc::Rename_Folder(UINT offset, LPCTSTR new_name)
{
	CString s1, s2, s3;
	int pos, len, diff;
	UINT i;

	s1 = *m_pPathList[offset].path;			// Get current folder path.
	len = s1.GetLength();					// And its length.
	s2 = s1;								// Extract plain folder name.
	Friendly_Name(s2);						
	diff = strlen(new_name) - s2.GetLength();	// Get difference in length.
	if (diff > 0)							// If new name is longer,
	{										// check resulting path lengths.
		for (i = 1; i < m_intPathCount; i++)
		{
			if (m_pPathList[i].path->Find(s1) == 0)
			{
				if ((m_intBasePathLen + m_pPathList[i].path->GetLength() + diff) > 200)
					return FALSE;
			}
		}
	}
	if (m_pPathList[offset].parent != 0)	// Is it not right off the root?
	{
		s2 = s1.Left(s1.GetLength() - 1);	// Strip trailing backslash
		pos = s2.ReverseFind('\\') + 1;		// Find last backslash.
		s3 = s1.Left(pos);					// Extract parent folder name.
		s3 += new_name;						// Append new folder name.
	}
	else									// Otherwise,
	{
		s3 = new_name;						// just set new folder name.
	}
	s3 += '\\';								// Append trailing backslash
	for (i = 1; i < m_intPathCount; i++)	// Walk the array.
	{
		if (i == offset)					// Target folder?
			*m_pPathList[i].path = s3;		// Just install the new name.
		else
		{
			if (m_pPathList[i].path->Find(s1) == 0)			// Child folder?
			{
				s2 = s3 + m_pPathList[i].path->Mid(len);	// Strip old parent, install new parent.
				*m_pPathList[i].path = s2;
			}
		}
	}
	SetModifiedFlag();		// Mark document at dirty.
	return TRUE;			// Success!
}

// Warns user before export if root folder is not selected.
BOOL CSUDoc::Do_Export_Warning()
{
	if (m_pPathList[0].selected == 1)
		return AfxMessageBox(IDP_WARN_EXPORT, MB_YESNO | MB_ICONQUESTION) == IDYES;
	else
		return TRUE;
}

// Writes non-fatal HTML error to log file.
void CSUDoc::Report_HTML_Err(CString & html, LPCTSTR err)
{
	CString msg;

	msg.Format("HTML Error in:\n%s\n%s. The line was skipped\n", html, err);
	m_pErrFile->WriteString(msg);
	m_bErrorsLogged = TRUE;
}

// Writes fatal error message to log file.
void CSUDoc::Report_Fatal_Error(LPCTSTR text)
{
	CString msg;

	msg.Format("Critical error:\n%s\nProcessing aborted\n", text);
	m_pErrFile->WriteString(msg);
	m_bErrorsLogged = TRUE;
}

// Deletes the path specified by the offset parm.
// Note that the UI ensures that this function will only
// be called if the target folder has no children, either
// folders or bookmarks, greatly simplifying the task.
void CSUDoc::Delete_Folder(UINT offset)
{
	PATH_ELEMENT *temp, *p1, *p2;
	UINT i;
	size_t size = sizeof(PATH_ELEMENT);

	delete m_pPathList[offset].path;		// Delete path string.
	--m_intPathCount;						// Adjust path count.
	if (offset < m_intPathCount)			// If deleted path was not the last one in the array,
	{
		p1 = &m_pPathList[offset];			// Close the gap in the array.
		p2 = &m_pPathList[offset + 1];
		memmove(p1, p2, (m_intPathCount - offset) * size);
	}
	// Reallocate the array.
	temp = (PATH_ELEMENT*) realloc(m_pPathList, size * m_intPathCount);
	ASSERT(temp != NULL);
	m_pPathList = temp;
	// Adjust parent offsets for path items.
	for (i = 0; i < m_intPathCount; i++)
	{
		if (m_pPathList[i].parent >= offset)
			--m_pPathList[i].parent;
	}
	// Adjust path offsets for bookmark items.
	for (i = 0; i < m_intItemCount; i++)
	{
		if (m_pDocElements[i].path_offset >= offset)
			--m_pDocElements[i].path_offset;
	}
}

// Initializes the error log file.
// Returns FALSE if file could not be created.
BOOL CSUDoc::Start_Error_Log()
{
	ASSERT(m_pErrFile == NULL);

	CTime time = CTime::GetCurrentTime();
	CString strDate;
	strDate.Format("SyncURLs import log created %s\n\n", time.Format("%c"));

	m_pErrFile = new CStdioFile;
	if (m_pErrFile != NULL)
	{
		if (m_pErrFile->Open(m_strErrLog, CFile::modeCreate | CFile::modeWrite | CFile::shareDenyWrite, NULL))
		{
			m_bErrorsLogged = FALSE;
			m_pErrFile->WriteString(strDate);
			return TRUE;
		}
		else
		{
			delete m_pErrFile;
			m_pErrFile = NULL;
		}
	}
	return FALSE;
}

// Closes the error log.
void CSUDoc::End_Error_Log()
{
	m_pErrFile->Close();
	delete m_pErrFile;
	m_pErrFile = NULL;
}

// Called after any import. Sorts the document and resolves duplicate names.
void CSUDoc::Do_Import_Cleanup()
{
	BOOL result;

	m_pStatDlg->Show_Message("Sorting...");
	qsort((void*) m_pDocElements, m_intItemCount, sizeof(DOC_ELEMENT), Compare_Items);
	m_pStatDlg->Show_Message("Resolving duplicate names...");
	result = Resolve_Dupes();
	if (result)
	{
		m_pStatDlg->Show_Message("Duplicate names resolved. Re-sorting...");
		qsort((void*) m_pDocElements, m_intItemCount, sizeof(DOC_ELEMENT), Compare_Items);
	}
}

// Presents query if errors encountered during import.
void CSUDoc::Query_Show_Log(BOOL bFatal)
{
	CString strMessage;

	if (bFatal)
		strMessage = "One or more critical errors occurred during import.";
	else
		strMessage =	"SyncURLs had to make some adjustments\n"
						"while importing your bookmarks.";
	strMessage += "\n\nTo see the Import Log, use the\n"
				  "\"Import Log\" command from the View Menu.";
	AfxMessageBox(strMessage, MB_ICONINFORMATION);
}


void CSUDoc::OnUpdateExportManual(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_intItemCount != 0);	
}

// Determine which browser(s) are present on the local system.
void CSUDoc::Detect_Browsers()
{
	HKEY key;
	HRESULT hResult;
	DWORD len = _MAX_PATH;
	DWORD data_type;
	unsigned char buff[_MAX_PATH];
	char buff2[_MAX_PATH];

	// Detect MSIE
	// Try to open the "User Shell Folders" key.
	hResult = RegOpenKeyEx(HKEY_CURRENT_USER, sz_IEKey, 0, KEY_QUERY_VALUE, &key);
	if (hResult == ERROR_SUCCESS)
	{
		// Try to read the "Favorites" data value.
		hResult = RegQueryValueEx(key, sz_IEData, 0, &data_type, buff, &len);
		RegCloseKey(key);
	}
	// If that failed,
	if (hResult != ERROR_SUCCESS)
	{
		//  try to open the "Shell Folders" key.
		hResult = RegOpenKeyEx(HKEY_CURRENT_USER, sz_IEAltKey, 0, KEY_QUERY_VALUE, &key);
		if (hResult == ERROR_SUCCESS)	// If successful,
		{
			// Try to read the "Favorites" data value.
			hResult = RegQueryValueEx(key, sz_IEData, 0, &data_type, buff, &len);
			RegCloseKey(key);
		}
	}
	// If value was retrieved,
	if (hResult == ERROR_SUCCESS)
	{
		// Check to see if e-vars need to be expanded.
		// (Only happens under NT -- Win 95 doesn't support the REG_EXPAND_SZ data type)
		if (data_type == REG_EXPAND_SZ)
		{
			// If so, expand e-vars, then set the value.
			ExpandEnvironmentStrings((const char *)buff, buff2, _MAX_PATH);
			m_strIEPath = buff2;
		}
		else
			// Otherwise, just set the value.
			m_strIEPath = buff;
		// Make sure the path ends in a backslash.
		if (m_strIEPath.Right(1) != "\\")
			m_strIEPath += "\\";
		m_bIEPresent = TRUE;
	}
	m_intBasePathLen = m_strIEPath.GetLength();

	// Detect Netscape
	len = _MAX_PATH;
	if (RegOpenKeyEx(HKEY_CURRENT_USER, sz_NetscapeKey, 0, KEY_QUERY_VALUE, &key) == ERROR_SUCCESS)
	{
		if (RegQueryValueEx(key, sz_NetscapeData, 0, &data_type, buff, &len) == ERROR_SUCCESS)
		{
			m_bNSPresent = TRUE;
			if (data_type == REG_EXPAND_SZ)
			{
				ExpandEnvironmentStrings((const char *)buff, buff2, _MAX_PATH);
				m_strNSPath = buff2;
			}
			else
				m_strNSPath = buff;
		}
		RegCloseKey(key);
	}

	// Detect Communicator
	len = MAX_PATH;
	if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, sz_CommunicatorKey, 0, KEY_QUERY_VALUE, &key) == ERROR_SUCCESS)
	{
		m_bNSCPresent = TRUE;
		RegCloseKey(key);
	}
	else
		m_bNSCPresent = FALSE;
	if (m_bNSCPresent)
	{
		CBrowsDlg dlg;
		dlg.m_pMyDoc = this;
		dlg.DoModal();
	}
	if (m_bNSCPresent && m_bNSPresent)
	{
		if (m_strNSPath.CompareNoCase(m_strNSCPath) == 0)
		{
			AfxMessageBox(IDP_NS_SHARED, MB_ICONINFORMATION);
			m_bNSPresent = FALSE;
			m_strNSCUserName += " [shared]";
		}
	}
}

void CSUDoc::OnUpdateExportNsc(CCmdUI* pCmdUI) 
{
	if (m_bNSCPresent)
	{
		CString strText;
		strText.Format("Netscape &4.x (%s)", m_strNSCUserName);
		pCmdUI->SetText(strText);
	}
	pCmdUI->Enable(m_bNSCPresent && (m_intItemCount > 0));
}

void CSUDoc::OnViewLog() 
{
	HINSTANCE hInst;
	CString strMessage;

	hInst = ShellExecute(NULL, "open", m_strErrLog, NULL, NULL, SW_SHOWNORMAL);
	if ((UINT) hInst <= 32)
	{
		strMessage.Format("Failed to launch external application\n"
			"Either there is no association for the .TXT extension,\n"
			"or the existing association is incomplete\n"
			"If you wish to view the error log manually,\n"
			"You'll find it at:\n%s", m_strErrLog);
		AfxMessageBox(strMessage, MB_ICONINFORMATION);
	}
}

void CSUDoc::OnUpdateViewLog(CCmdUI* pCmdUI) 
{
	CFile file;

	if (!file.Open(m_strErrLog, CFile::modeRead, NULL))
	{
		file.Close();
		pCmdUI->Enable(FALSE);
	}
	else
		pCmdUI->Enable(TRUE);
}

