// SUView.cpp : implementation of the CSUView class
//
//
// SyncURLs version 1.0
// Copyright 1998 Ziff-Davis Inc.
// First published in PC Magazine, US Edition
// January 6, 1998
// Written by Gregory A. Wolking
//

#include "stdafx.h"
#include "SyncURLs.h"
#include <afxpriv.h>

#include "MyTree.h"
#include "MyList.h"
#include "PropDlg.h"
#include "WeedDlg.h"
#include "StatDlg.h"
#include "FindDlg.h"
#include "ScrllDlg.h"
#include "RenDlg.h"
#include "SUDoc.h"
#include "SUView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


/////////////////////////////////////////////////////////////////////////////
// CSUView

IMPLEMENT_DYNCREATE(CSUView, CFormView)

BEGIN_MESSAGE_MAP(CSUView, CFormView)
	//{{AFX_MSG_MAP(CSUView)
	ON_WM_CREATE()
	ON_COMMAND(ID_EDIT_RENAME, OnEditRename)
	ON_UPDATE_COMMAND_UI(ID_EDIT_RENAME, OnUpdateEditRename)
	ON_NOTIFY(TVN_SELCHANGED, IDC_tvnFolders, OnSelchangedtvnFolders)
	ON_NOTIFY(NM_RCLICK, IDC_tvnFolders, OnRclicktvnFolders)
	ON_NOTIFY(TVN_BEGINDRAG, IDC_tvnFolders, OnBegindragtvnFolders)
	ON_WM_LBUTTONUP()
	ON_WM_MOUSEMOVE()
	ON_COMMAND(ID_EDIT_DEL, OnEditDel)
	ON_UPDATE_COMMAND_UI(ID_EDIT_DEL, OnUpdateEditDel)
	ON_COMMAND(ID_OPT_CONFIRM_BOOKMARK, OnOptConfirmBookmark)
	ON_UPDATE_COMMAND_UI(ID_OPT_CONFIRM_BOOKMARK, OnUpdateOptConfirmBookmark)
	ON_NOTIFY(NM_CLICK, IDC_tvnFolders, OnClicktvnFolders)
	ON_COMMAND(ID_OPT_CONFIRM_FOLDER, OnOptConfirmFolder)
	ON_UPDATE_COMMAND_UI(ID_OPT_CONFIRM_FOLDER, OnUpdateOptConfirmFolder)
	ON_COMMAND(ID_EDIT_NEW, OnEditNew)
	ON_COMMAND(ID_VIEW_PROPERTIES, OnViewProperties)
	ON_UPDATE_COMMAND_UI(ID_VIEW_PROPERTIES, OnUpdateViewProperties)
	ON_COMMAND(ID_EDIT_WEED, OnEditWeed)
	ON_UPDATE_COMMAND_UI(ID_EDIT_WEED, OnUpdateEditWeed)
	ON_WM_RBUTTONDOWN()
	ON_COMMAND(ID_VIEW_EXPAND, OnViewExpand)
	ON_COMMAND(ID_VIEW_COLLAPSE, OnViewCollapse)
	ON_COMMAND(ID_OPT_AUTO, OnOptAuto)
	ON_COMMAND(ID_EDIT_FIND, OnEditFind)
	ON_COMMAND(ID_EDIT_FIND_NEXT, OnEditFindNext)
	ON_UPDATE_COMMAND_UI(ID_EDIT_FIND_NEXT, OnUpdateEditFindNext)
	ON_UPDATE_COMMAND_UI(ID_EDIT_FIND, OnUpdateEditFind)
	ON_COMMAND(ID_EXPORT_MANUAL, OnExportManual)
	ON_COMMAND(ID_EXPORT_NS, OnExportNs)
	ON_COMMAND(ID_EXPORT_MSIE, OnExportMsie)
	ON_COMMAND(ID_EXPORT_BOTH, OnExportBoth)
	ON_COMMAND(ID_EXPORT_NSC, OnExportNsc)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CSUView construction/destruction

CSUView::CSUView()
	: CFormView(CSUView::IDD)
{
	//{{AFX_DATA_INIT(CSUView)
	//}}AFX_DATA_INIT
	// TODO: add construction code here
	m_intSearchIn = 0;
	m_strSearchText.Empty();
	m_bSearchCase = FALSE;
	m_bSearchStart = FALSE;
	m_pFile = NULL;
}

CSUView::~CSUView()
{
	CWinApp* pMyApp = AfxGetApp();
	pMyApp->WriteProfileInt("Settings", "ConfirmBookmarkDel", m_bConfirmBookmarkDelete);
	pMyApp->WriteProfileInt("Settings", "ConfirmFolderDel", m_bConfirmFolderDelete);
}

void CSUView::DoDataExchange(CDataExchange* pDX)
{
	CFormView::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CSUView)
	DDX_Control(pDX, IDC_tvnFolders, m_tvnFolders);
	//}}AFX_DATA_MAP
}

BOOL CSUView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CFormView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CSUView diagnostics

#ifdef _DEBUG
void CSUView::AssertValid() const
{
	CFormView::AssertValid();
}

void CSUView::Dump(CDumpContext& dc) const
{
	CFormView::Dump(dc);
}

CSUDoc* CSUView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CSUDoc)));
	return (CSUDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CSUView message handlers

/////////////////////////
// CFormView handler overrides

void CSUView::OnInitialUpdate() 
{
	CFormView::OnInitialUpdate();
	
	GetParentFrame()->RecalcLayout();
	SetScaleToFitSize(CSize(1,1));
	if (m_imgTree.m_hImageList == NULL)							// If image list doesn't exist yet,
	{
		m_imgTree.Create(IDB_BITMAP2, 16, 0, 0x00FFFFFF);		// Create it.
		m_tvnFolders.SetImageList(&m_imgTree, TVSIL_NORMAL);	// Attach it to the tree control.
		m_imgState.Create(IDB_BITMAP1, 16, 0, 0x00FFFFFF);
		m_tvnFolders.SetImageList(&m_imgState, TVSIL_STATE);
	}
}

void CSUView::OnDraw(CDC* pDC) 
{
	if (m_tvnFolders.m_hWnd == NULL)
		return;
	CRect wnd_rect;

	GetWindowRect(wnd_rect);				// Get window coordinates.
	ScreenToClient(wnd_rect);				// Convert to client coordinates.
	m_tvnFolders.MoveWindow(wnd_rect);		// Adjust tree control to fill the window.
}

void CSUView::OnUpdate(CView* pSender, LPARAM lHint, CObject* pHint) 
{
	BeginWaitCursor();								// Let user know we're working.
	m_pMyDoc = (CSUDoc*) GetDocument();				// Make sure pointer to current document is valid.
	m_tvnFolders.DeleteAllItems();					// Zap the tree control.
	m_pMyDoc->Reset_Path_Flags();					// Prepare for recursion.
	Process_Path(0, TVI_ROOT);						// Fill the tree control.
	m_hCurItem = m_tvnFolders.GetRootItem();		// Set current item handle to root folder
	m_tvnFolders.Expand(m_hCurItem, TVE_EXPAND);	// Expand the root folder.
	// If hint parameter is non-zero, update was due to a folder move,
	if (lHint)
		// so make sure the newly-moved folder is visible.
		Locate_Folder(m_hCurItem, lHint);
	EndWaitCursor();								// Restore normal cursor.
}

int CSUView::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CFormView::OnCreate(lpCreateStruct) == -1)
		return -1;

	CWinApp* pMyApp = AfxGetApp();
	m_hCurItem = NULL;
	m_hDragSource = NULL;
	m_hDragDest = NULL;
	m_pMyDoc = NULL;
	m_bIsDragging = FALSE;
	// Retrieve settings from Registry.
	m_bConfirmBookmarkDelete = pMyApp->GetProfileInt("Settings", "ConfirmBookmarkDel", TRUE);
	m_bConfirmFolderDelete = pMyApp->GetProfileInt("Settings", "ConfirmFolderDel", TRUE);
	// Clean up any invalid numeric Registry data.
	m_intScrollRate = pMyApp->GetProfileInt("Settings", "ScrollRate", 0);
	if (m_intScrollRate > 100)
	{
		m_intScrollRate = 100;
		pMyApp->WriteProfileInt("Settings", "ScrollRate", 100);
	}
	if (m_intScrollRate < 0)
	{
		m_intScrollRate = 0;
		pMyApp->WriteProfileInt("Settings", "ScrollRate", 0);
	}
	return 0;
}

////////////////////////////////
// Menu command and UI handlers.

// "Edit" menu, "Rename" command.
void CSUView::OnEditRename() 
{
	HTREEITEM hCurItem, hParentItem, hWorkItem;
	CString strOldName, strNewName, strWork;
	BOOL bIsBookmark, bDupe;
	char szBuff[_MAX_PATH];
	TV_ITEM tvi;
	UINT nItemIndex;

	hCurItem = m_tvnFolders.GetSelectedItem();
	if (hCurItem == NULL)
		return;
	hParentItem = m_tvnFolders.GetParentItem(hCurItem);
	tvi.mask = TVIF_TEXT | TVIF_IMAGE | TVIF_PARAM;
	tvi.hItem = hCurItem;
	tvi.cchTextMax = sizeof(szBuff);
	tvi.pszText = szBuff;
	m_tvnFolders.GetItem(&tvi);
	bIsBookmark = (tvi.iImage == 1);
	nItemIndex = tvi.lParam;
	strOldName = szBuff;
	CRenDlg dlg(this);
	dlg.m_txtNewName = strOldName;
	if (bIsBookmark)
		dlg.m_strType = "Bookmark";
	else
		dlg.m_strType = "Folder";
	// Loop repeats until either a valid name is given or the user cancels.
	while (TRUE)
	{
		if (dlg.DoModal() != IDOK)
			return;
		strNewName = dlg.m_txtNewName;
		strNewName.TrimRight();
		strNewName.TrimLeft();
		if (strNewName.IsEmpty())	// Error if new name is blank.
		{
			AfxMessageBox("Name cannot be blank", MB_ICONINFORMATION);
			dlg.m_txtNewName = strOldName;
			continue;
		}
		// Error if name begings with a "."
		if (strNewName.Left(1) == '.')
		{
			AfxMessageBox("Item names cannot begin with a period.", MB_ICONINFORMATION);
			continue;
		}
		// Clean up illegal characters.
		m_pMyDoc->Fix_FileName(strNewName);
		// If names are identical after all of this, there's nothing to do!
		if (strOldName == strNewName)
			return;
		// For folders, make sure that name doesn't have the ".URL" extension.
		if (!bIsBookmark)
		{
			strWork = strNewName.Right(4);
			if (strWork.CompareNoCase(".url") == 0)
			{
				AfxMessageBox("Folder names cannot have the .URL extension", MB_ICONINFORMATION);
				dlg.m_txtNewName = strNewName;
				continue;
			}
		}
		// Check for existing item of the same type with the same name.
		hWorkItem = m_tvnFolders.GetChildItem(hParentItem);
		bDupe = FALSE;
		do
		{
			if (hWorkItem != hCurItem)
			{
				tvi.hItem = hWorkItem;
				m_tvnFolders.GetItem(&tvi);
				if (bIsBookmark && tvi.iImage != 1)
					continue;
				if (!bIsBookmark && tvi.iImage == 1)
					continue;
				if (strNewName.CompareNoCase(szBuff) == 0)
				{
					bDupe = TRUE;
					break;
				}
			}
		} while ((hWorkItem = m_tvnFolders.GetNextSiblingItem(hWorkItem)) != NULL);
		if (bDupe)	// Error and repeat if dupe item found.
		{
			AfxMessageBox("New name must be unique.", MB_ICONINFORMATION);
			continue;
		}
		// If we got this far, we're ready to rename.
		if (bIsBookmark)	// Handle bookmark.
		{
			*m_pMyDoc->m_pDocElements[nItemIndex].page = strNewName;	// Update the document.
			m_pMyDoc->SetModifiedFlag();
			m_tvnFolders.SetItemText(hCurItem, strNewName);		// Update the item's text.
			m_hDragSource = hCurItem;							// Move the item to its own
			m_hDragDest = hParentItem;							// Parent folder. This puts the
			Move_Bookmark(TRUE);								// renamed item back in sorted order.
			break;	// All done, so exit the loop.
		}
		// For folders, make sure we can update it. (If resulting path
		// would be too long, Rename_Folder() returns false).
		if (!m_pMyDoc->Rename_Folder(nItemIndex, strNewName))
		{
			AfxMessageBox("Path too long.\nYou must use a shorter name for this folder", MB_ICONINFORMATION);
			continue;
		}
		// Update folder item's text.
		m_tvnFolders.SetItemText(hCurItem, strNewName);
		// Move it to its own parent so it goes into proper sorted order.
		m_hDragSource = hCurItem;
		m_hDragDest = hParentItem;
		Move_Folder(TRUE);
		break;
	}
}

void CSUView::OnUpdateEditRename(CCmdUI* pCmdUI) 
{
	BOOL status = FALSE;
	TV_ITEM my_item;
	CString msg;
	
	if (m_hCurItem)
	{
		my_item.hItem = m_hCurItem;
		my_item.mask = TVIF_IMAGE;
		m_tvnFolders.GetItem(&my_item);
		switch(my_item.iImage)
		{
		case 0:
			msg = "&Rename folder\tCtrl+R";
			status = TRUE;
			break;
		case 2:
			msg = "&Rename folder\tCtrl+R";
			break;
		case 1:
			msg = "&Rename bookmark\tCtrl+R";
			status = TRUE;
			break;
		case 3:
			msg = "&Rename bookmark\tCtrl+R";
		}
	}
	pCmdUI->SetText(msg);
	pCmdUI->Enable(status);
}

// "Edit" menu, "Delete" command.
void CSUView::OnEditDel() 
{
	HTREEITEM hItem, hParent;
	UINT offset, old_state;
	TV_ITEM info;
	BOOL all;

	hItem = m_tvnFolders.GetSelectedItem();				// Get handle to current hItem.
	hParent = m_tvnFolders.GetParentItem(hItem);
	if (IsBookmark(hItem))
	{
		if (m_bConfirmBookmarkDelete)						// Give confirmation prompt if appropriate.
		{
			if (AfxMessageBox("Are you sure you want to delete this bookmark?", MB_YESNO | MB_ICONQUESTION) != IDYES)
				return;
		}
		BeginWaitCursor();									// Put up the busy cursor
		m_nDeletedItem = m_tvnFolders.GetItemData(hItem);	// Get hItem's offset into the document.
		m_pMyDoc->Delete_Item(m_nDeletedItem);				// Remove it from the document.
		m_tvnFolders.DeleteItem(hItem);						// Remove the hItem from the tree.
		Adjust_Bookmarks(m_tvnFolders.GetRootItem());		// Adjust offsets for other items.
		EndWaitCursor();									// Restore normal cursor.
	}
	else
	{
		if (m_bConfirmFolderDelete)
		{
			if (AfxMessageBox("Are you sure you want to delete this folder?", MB_YESNO | MB_ICONQUESTION) != IDYES)
				return;
		}
		BeginWaitCursor();
		offset = m_tvnFolders.GetItemData(hItem);
		m_pMyDoc->Delete_Folder(offset);
		m_tvnFolders.DeleteItem(hItem);
		Adjust_Folder_Offsets(m_tvnFolders.GetRootItem(), offset);
		info.mask = TVIF_CHILDREN | TVIF_STATE | TVIF_PARAM;
		info.hItem = hParent;
		m_tvnFolders.GetItem(&info);
		old_state = (info.state & TVIS_STATEIMAGEMASK) >> 12;
		offset = (UINT) info.lParam;
		if (old_state == 3)					// If parent folder was partially selected,
		{
			if (info.cChildren == 0)		// and it has no remaining children,
				old_state = 2;				// it becomes fully selected.
			else							// Otherwise, check status of remaining children.
			{
				all = TRUE;					// Default = All children selected.
				hItem = m_tvnFolders.GetChildItem(hParent);	// Get handle of first child item.
				info.mask = TVIF_STATE | TVIF_IMAGE;		// Mask for info required for each item.
				do
				{
					info.hItem = hItem;						// Set handle.
					m_tvnFolders.GetItem(&info);			// Retrieve info.
					if (info.iImage != 1)					// Is item a folder? 
					{
						if ((info.state & TVIS_STATEIMAGEMASK) != 0x2000)	// Is it not fully selected?
						{
							all = FALSE;					// Then not all children are selected.
							break;							// Done looking.
						}
					}
					hItem = m_tvnFolders.GetNextSiblingItem(hItem);	// Next item.
				} while (hItem != NULL);	// Loop until no items left.
				if (all)				// If all children were selected,
					old_state = 2;		// parent becomes fully selected.
			}
			if (old_state != 3)			// If selection state changed,
			{
				m_pMyDoc->m_pPathList[offset].selected = old_state;	// Update the document.
				info.mask = TVIF_STATE;								// Update the item.
				info.state = old_state << 12;
				info.stateMask = TVIS_STATEIMAGEMASK;
				info.hItem = hParent;
				m_tvnFolders.SetItem(&info);
				Set_State_Up(hParent);								// Update the rest of the tree.
			}
		}
		EndWaitCursor();
	}
}

void CSUView::OnUpdateEditDel(CCmdUI* pCmdUI) 
{
	BOOL status = FALSE;
	HTREEITEM hItem;
	TV_ITEM info;

	if (m_pMyDoc->m_intItemCount)
	{
		hItem = m_tvnFolders.GetSelectedItem();
		if (hItem != NULL)
		{
			info.mask = TVIF_IMAGE | TVIF_CHILDREN;
			info.hItem = hItem;
			m_tvnFolders.GetItem(&info);
			status = ((info.iImage == 1) || (info.iImage == 0 && info.cChildren == 0));
			if (info.iImage == 1)
				pCmdUI->SetText("&Delete bookmark\tDel");
			else
				pCmdUI->SetText("&Delete folder\tDel");
		}
	}
	pCmdUI->Enable(status);
}

void CSUView::OnOptConfirmBookmark() 
{
	m_bConfirmBookmarkDelete = !m_bConfirmBookmarkDelete;	
}

void CSUView::OnUpdateOptConfirmBookmark(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(m_bConfirmBookmarkDelete);	
}

void CSUView::OnOptConfirmFolder() 
{
	m_bConfirmFolderDelete = !m_bConfirmFolderDelete;	
}

void CSUView::OnUpdateOptConfirmFolder(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(m_bConfirmFolderDelete);	
}

// "Edit" menu, "New folder" command.
void CSUView::OnEditNew() 
{
	HTREEITEM hParent, hCurItem;
	TV_ITEM info, info2;
	TV_INSERTSTRUCT is;
	char buff[_MAX_PATH];
	CString strNewPath, strNewName;
	UINT nOffset, nSuffix = 0;
	BOOL valid;

	hCurItem = m_tvnFolders.GetSelectedItem();
	if (hCurItem == NULL)
		return;
	info.mask = TVIF_IMAGE;
	info.hItem = hCurItem;
	m_tvnFolders.GetItem(&info);
	if (info.iImage == 1)
		hParent = m_tvnFolders.GetParentItem(hCurItem);
	else
		hParent = hCurItem;
	ASSERT (hParent != NULL);
	info.mask = TVIF_CHILDREN | TVIF_PARAM;
	info.hItem = hParent;
	m_tvnFolders.GetItem(&info);
	nOffset = (UINT) info.lParam;
	strNewPath = *m_pMyDoc->m_pPathList[nOffset].path;
	if ((m_pMyDoc->m_intBasePathLen + strNewPath.GetLength()) > 180)
	{
		AfxMessageBox("Path to this folder is too long.\n"
		"Cannot create a new folder here.", MB_ICONINFORMATION);
		return;
	}
	do
	{
		if (nSuffix)
			strNewName.Format("New Folder (%u)", nSuffix);
		else
			strNewName = "New Folder";
		if (info.cChildren == 0)
			break;
		hCurItem = m_tvnFolders.GetChildItem(hParent);
		valid = TRUE;
		info2.mask = TVIF_IMAGE | TVIF_TEXT;
		info2.pszText = buff;
		info2.cchTextMax = sizeof(buff);
		while (hCurItem != NULL)
		{
			info2.hItem = hCurItem;
			m_tvnFolders.GetItem(&info2);
			if (info2.iImage != 1)
			{
				if (strNewName.CompareNoCase(info2.pszText) == 0)
				{
					valid = FALSE;
					break;
				}
			}
			hCurItem = m_tvnFolders.GetNextSiblingItem(hCurItem);
		}
		++nSuffix;
	} while (!valid);
	strNewPath += strNewName;
	strNewPath += '\\';
	if (!m_pMyDoc->Add_Path(strNewPath, FALSE, &nOffset))
	{
		AfxMessageBox("Internal error -- could not create path item", MB_ICONINFORMATION);
		return;
	}
	is.hParent = hParent;
	lstrcpyn(buff, strNewName, sizeof(buff));
	is.hInsertAfter = TVI_SORT;
	is.item.mask = TVIF_IMAGE | TVIF_STATE | TVIF_TEXT | TVIF_PARAM;
	is.item.iImage = 0;
	is.item.cchTextMax = strNewName.GetLength() + 1;
	is.item.pszText = buff;
	is.item.state = (2 << 12);
	is.item.stateMask = TVIS_STATEIMAGEMASK;
	is.item.lParam = (LPARAM) nOffset;
	hCurItem = m_tvnFolders.InsertItem(&is);
	ASSERT (hCurItem != NULL);
	m_tvnFolders.EnsureVisible(hCurItem);
	Set_State_Up(hCurItem);
	m_tvnFolders.SelectItem(hCurItem);
}

void CSUView::OnEditWeed() 
{
	if (Weed_Duplicates())
		OnUpdate(NULL, 0, NULL);
}

void CSUView::OnUpdateEditWeed(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_pMyDoc->m_intItemCount >= 2);
}

// Export Menu commands.
// Export to user-selected HTML file.
void CSUView::OnExportManual() 
{
	CString strOldName, strOldTitle;

	if (!m_pMyDoc->Do_Export_Warning())	// Give user a chance to bail if no folders are selected.
		return;
	// Create a Save As file dialog with appropriate title and filter settings.
	CFileDialog dlg(FALSE, ".htm", "Bookmark.htm", OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT,
		"HTML Files (*.htm; *.html)|*.htm; *.html||", NULL);
	dlg.m_ofn.lpstrTitle = "Export HTML File";
	// Launch the File dialog.
	if (dlg.DoModal() != IDOK)
		return;		// Done if user didn't say OK.
	// Save existing title and filename strings.
	strOldName = m_pMyDoc->m_strNSPath;
	strOldTitle = m_pMyDoc->m_strNSPageTitle;
	// Install new title and filename strings.
	m_pMyDoc->m_strNSPath = dlg.GetPathName();
	m_pMyDoc->m_strNSPageTitle = "Bookmarks exported by SyncURLs";
	// Export to the specified file.
	Write_NS_File(FALSE);
	// Restore original title and filename strings.
	m_pMyDoc->m_strNSPath = strOldName;
	m_pMyDoc->m_strNSPageTitle = strOldTitle;
}

// Exports to Netscape 3.x only.
void CSUView::OnExportNs() 
{
	if (!m_pMyDoc->Do_Export_Warning())	// Give user a chance to bail if no folders are selected.
		return;
	Write_NS_File();	
}

// Exports to all detected browsers.
void CSUView::OnExportBoth() 
{
	if (!m_pMyDoc->Do_Export_Warning())		// Warn if no folders selected.
		return;
	if (m_pMyDoc->m_bNSPresent)				// NS 3.x detected?
		Write_NS_File();					// Export if so.
	if (m_pMyDoc->m_bNSCPresent)			// NS 4.x detected?
	{
		CString strOldPath(m_pMyDoc->m_strNSPath);			// Save NS 3.x path.
		m_pMyDoc->m_strNSPath = m_pMyDoc->m_strNSCPath;		// Install NS 4.x path.
		Write_NS_File();									// Export.
		m_pMyDoc->m_strNSPath = strOldPath;					// Restore NS 3.x path.
	}
	m_pMyDoc->Do_MSIE_Export();				// Export to MSIE, if detected.
}

// Exports to MSIE only.
void CSUView::OnExportMsie() 
{
	if (!m_pMyDoc->Do_Export_Warning())
		return;
	m_pMyDoc->Do_MSIE_Export();
}

// Exports to NS 4.x only.
void CSUView::OnExportNsc() 
{
	if (!m_pMyDoc->Do_Export_Warning())	// Give user a chance to bail if no folders are selected.
		return;
	CString strOldPath(m_pMyDoc->m_strNSPath);
	m_pMyDoc->m_strNSPath = m_pMyDoc->m_strNSCPath;
	Write_NS_File();
	m_pMyDoc->m_strNSPath = strOldPath;
}

void CSUView::OnViewExpand() 
{
	Expand_All(NULL, TRUE);
}

void CSUView::OnViewCollapse() 
{
	Expand_All(NULL, FALSE);
}

void CSUView::OnOptAuto() 
{
	CScrllDlg dlg;

	dlg.m_intScrollRate = 100 - m_intScrollRate;
	if (dlg.DoModal() != IDOK)
		return;
	m_intScrollRate = 100 - dlg.m_intScrollRate;
	AfxGetApp()->WriteProfileInt("Settings", "ScrollRate", m_intScrollRate);
}


void CSUView::OnUpdateEditFindNext(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(!m_strSearchText.IsEmpty() && m_pMyDoc->m_intItemCount > 0);	
}

void CSUView::OnUpdateEditFind(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_pMyDoc->m_intItemCount > 0);	
}

void CSUView::OnEditFind() 
{
	CFindDlg dlg;							// Create search criteria dialog.
	HTREEITEM hItem;

	dlg.m_bIgnoreCase = m_bSearchCase;		// Load it with current criteria.
	dlg.m_intLook = m_intSearchIn;
	dlg.m_strTarget = m_strSearchText;
	dlg.m_intSearchStart = m_bSearchStart;
	if (dlg.DoModal() != IDOK)				// Launch the dialog.
		return;								// Done if user cancelled.
	m_bSearchCase = dlg.m_bIgnoreCase;		// Store new criteria.
	m_intSearchIn = dlg.m_intLook;
	m_strSearchText = dlg.m_strTarget;
	m_bSearchStart = dlg.m_intSearchStart;
	if (m_bSearchStart)
		hItem = m_tvnFolders.GetSelectedItem();
	else
		hItem = m_tvnFolders.GetRootItem();
	if (!Search_Tree(hItem))				// Do the search.
		AfxMessageBox(IDP_NOT_FOUND, MB_ICONINFORMATION);	// Error message if appropriate.
}

void CSUView::OnEditFindNext() 
{
	HTREEITEM hItem;

	hItem = m_tvnFolders.GetSelectedItem();
	hItem = Next_Tree_Item(hItem);
	if (!Search_Tree(hItem))					// Continue existing search.
		AfxMessageBox(IDP_NOT_FOUND, MB_ICONINFORMATION);	// Error message if appropriate.
}

// View | Properties command.
// NOTE: UI enables this command only if a bookmark is selected in the tree.
void CSUView::OnViewProperties() 
{
	CPropDlg dlg;
	UINT nOffset;
	HTREEITEM hItem;
	DOC_ELEMENT* pDocItem;

	hItem = m_tvnFolders.GetSelectedItem();							// Get index of current item.
	if (hItem == NULL)												// Nothing to do if no item selected.
		return;
	nOffset = m_tvnFolders.GetItemData(hItem);						// Get array offset of item.
	pDocItem = &m_pMyDoc->m_pDocElements[nOffset];					// Get pointer to document item.
	dlg.m_strPageName = *pDocItem->page;							// Get page name.
	dlg.m_strURL = *pDocItem->url;									// Get URL.
	Format_Date_Field(pDocItem->add_date, dlg.m_strAddDate);		// Retrieve and format date fields.
	Format_Date_Field(pDocItem->mod_date, dlg.m_strModDate);
	Format_Date_Field(pDocItem->visit_date, dlg.m_strVisitDate);
	if (dlg.DoModal() == IDOK)					// Launch dialog. If user said "OK",
	{
		if (*pDocItem->url != dlg.m_strURL)		// see if URL string changed. If it did,
		{
			*pDocItem->url = dlg.m_strURL;		// install the updated string and
			m_pMyDoc->SetModifiedFlag();		// mark the document as "dirty'.
		}
	}
}

void CSUView::OnUpdateViewProperties(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(IsBookmark(m_hCurItem));
}


//////////////////////////////////
// Control notification handlers.

void CSUView::OnSelchangedtvnFolders(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_TREEVIEW* pNMTreeView = (NM_TREEVIEW*)pNMHDR;
	
	m_hCurItem = pNMTreeView->itemNew.hItem;
	*pResult = 0;
}

// Provides right-click pop-up menu for the tree control.
void CSUView::OnRclicktvnFolders(NMHDR* pNMHDR, LRESULT* pResult) 
{
	CPoint point;
	HTREEITEM item;
	UINT flags;
	TV_ITEM item_info;

	GetCursorPos(&point);
	ScreenToClient(&point);
	item = m_tvnFolders.HitTest(point, &flags);
	if (item != NULL)
	{
		item_info.mask = TVIF_IMAGE | TVIF_CHILDREN;
		item_info.hItem = item;
		m_tvnFolders.GetItem(&item_info);
		ClientToScreen(&point);				// Get client coordinates for menu position.
		m_tvnFolders.SelectItem(item);		// Make sure item is selected.
		CMenu menu;							// Create the popup menu.
		menu.CreatePopupMenu();
		if (item_info.iImage == 1)			// Build menu based on item type.
		{
			menu.AppendMenu(MF_STRING, ID_EDIT_RENAME, "&Rename Bookmark");
			menu.AppendMenu(MF_STRING, ID_EDIT_DEL, "&Delete Bookmark");
			menu.AppendMenu(MF_SEPARATOR);
		}
		else
		{
			if (item_info.iImage != 2)
			{
				menu.AppendMenu(MF_STRING, ID_EDIT_RENAME, "&Rename Folder");
				if (item_info.iImage == 0 && item_info.cChildren == 0)
					menu.AppendMenu(MF_STRING, ID_EDIT_DEL, "&Delete Folder");
				menu.AppendMenu(MF_SEPARATOR);
			}
		}
		menu.AppendMenu(MF_STRING, ID_EDIT_NEW, "&New folder");
		if (item_info.iImage == 1)
		{
			menu.AppendMenu(MF_SEPARATOR);
			menu.AppendMenu(MF_STRING, ID_VIEW_PROPERTIES, "&Properties");
		}
		// Launch the menu.
		menu.TrackPopupMenu(TPM_LEFTALIGN | TPM_LEFTBUTTON, point.x, point.y, GetParentFrame(), NULL);
	}
	*pResult = 0;
}

// Initiates bookmark/folder drag.
void CSUView::OnBegindragtvnFolders(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_TREEVIEW* pNMTreeView = (NM_TREEVIEW*)pNMHDR;
	CPoint point;
	HTREEITEM drag_item;

	point = pNMTreeView->ptDrag;
	drag_item = pNMTreeView->itemNew.hItem;
	if (!IsLocked(drag_item))
	{
		m_bIsDragging = TRUE;
		m_hDragSource = drag_item;
		m_hDragDest = NULL;
		m_tvnFolders.SelectItem(drag_item);
		m_pDragImage = m_tvnFolders.CreateDragImage(drag_item);
		m_pDragImage->DragShowNolock(TRUE);
		m_pDragImage->SetDragCursorImage(0, CPoint(0,0));
		m_pDragImage->BeginDrag(0, CPoint(0,0));
		m_pDragImage->DragMove(point);
		m_pDragImage->DragEnter(this, point);
		SetCapture();
	}
	else
	{
		AfxMessageBox("Existing MSIE folders cannot be moved", MB_ICONINFORMATION);
		m_tvnFolders.SelectItem(drag_item);
		m_hCurItem = drag_item;
	}
	*pResult = 0;
}

// Handles mouse click in the tree control.
void CSUView::OnClicktvnFolders(NMHDR* pNMHDR, LRESULT* pResult) 
{
	CPoint point;
	HTREEITEM hItem;
	UINT nFlags;
	UINT nState;
	TV_ITEM tvi;
	
	GetCursorPos(&point);							// Get mouse position.
	ScreenToClient(&point);							// Convert to client coordinates.
	hItem = m_tvnFolders.HitTest(point, &nFlags);	// Ask the tree control what was clicked on.
	if (hItem != NULL)								// Nothing to do if not an hItem.
	{
		if (nFlags & TVHT_ONITEMSTATEICON)		// If it was the state image (selection box)
		{
			tvi.mask = TVIF_STATE;				// Get item state.
			tvi.hItem = hItem;
			m_tvnFolders.GetItem(&tvi);
			nState = tvi.state >> 12;			// Adjust value.
			if (nState > 1)						// If already checked,
				nState = 1;						// clear it.
			else								// Otherwise,
				nState = 2;						// check it.
			Set_State_Down(hItem, nState > 1);	// Set selection state for children.
			Set_State_Up(hItem);				// Adjust selection state for parents.
		}
	}
	*pResult = 0;
}


///////////////////////////////
// Mouse event handlers.

// Handles "drop" part of drag & drop.
void CSUView::OnLButtonUp(UINT nFlags, CPoint point) 
{
	UINT flags;

	if (m_bIsDragging)							// Nothing to do unless user is dragging an item.
	{
		// Find out which item (if any) is under the mouse.
		m_hDragDest = m_tvnFolders.HitTest(point, &flags);
		m_pDragImage->DragLeave(this);			// Erase the drag image from the screen.
		m_pDragImage->EndDrag();				// End the drag operation.
		delete m_pDragImage;					// Delete the drag image.
		m_pDragImage = NULL;					// NULL its pointer.
		ReleaseCapture();						// Release mouse capture.
		m_bIsDragging = FALSE;					// Clear our flag.
		m_tvnFolders.SelectDropTarget(NULL);	// Clear tree control's drop target selection.
		if (m_hDragDest != NULL)				// Nothing to do if user didn't drop on an item.
		{
			if (m_hDragSource != m_hDragDest)		// If user dropped an item on itself, nothing to do!
			{
				m_tvnFolders.SelectItem(m_hDragDest);	// Select the destination item.
				if (IsFolder(m_hDragSource))			// Call appropriate move routine based on item type.
					Move_Folder();
				else
					Move_Bookmark();
			}
		}
	}
	CFormView::OnLButtonUp(nFlags, point);
}

// RMB pressed. Used to cancel a drag operation.
void CSUView::OnRButtonDown(UINT nFlags, CPoint point) 
{
	if (m_bIsDragging)						// If user is dragging an item,
	{
		m_bIsDragging = FALSE;				// Reset flag.
		m_pDragImage->DragLeave(this);		// Erase drag image from screen.
		m_pDragImage->EndDrag();			// End the drag.
		delete m_pDragImage;				// Delete the drag image.
		m_pDragImage = NULL;				// NULL its pointer.
		ReleaseCapture();					// Release mouse capture.
		m_tvnFolders.SelectDropTarget(NULL);	// Clear possible drop target selection.
		m_tvnFolders.EnsureVisible(m_hDragSource);	// Make sure item being dragged is visible.
	}
	CFormView::OnRButtonDown(nFlags, point);
}

// Handles dragging.
void CSUView::OnMouseMove(UINT nFlags, CPoint point) 
{
	HTREEITEM hitem;
	UINT flags;
	UINT msg = 0;
	WPARAM wParam;
	CPoint ptAnchor, ptCurrent;
	MSG message;

	if (m_bIsDragging)												// If user is dragging an item,
	{
		m_pDragImage->DragMove(point);								// Move the drag image.
		if ((hitem = m_tvnFolders.HitTest(point, &flags)) != NULL)	// Is mouse on a tree item?
		{
			m_pDragImage->DragLeave(this);					// If so, erase the drag image.
			m_tvnFolders.SelectDropTarget(hitem);			// Mark the target item.
			m_pDragImage->DragEnter(this, point);			// Redraw the drag image.
		}
		else	// Otherwise
		{
			// See if mouse is outside the tree control.
			if (flags & TVHT_ABOVE)		// Above the control?
			{
				msg = WM_VSCROLL;		// Vertical scrollbar
				wParam = SB_LINEUP;		// up one line.
			}
			else
			{
				if (flags & TVHT_BELOW)		// Below the control?
				{
					msg = WM_VSCROLL;		// Vertical scrollbar
					wParam = SB_LINEDOWN;	// down one line.
				}
				else
				{
					if (flags & TVHT_TOLEFT)	// Left of control?
					{
						msg = WM_HSCROLL;		// Horizontal scrollbar
						wParam = SB_LINELEFT;	// one line to the left.
					}
					else
					{
						if (flags & TVHT_TORIGHT)	// Right of control?
						{
							msg = WM_HSCROLL;		// Horizontal scrollbar
							wParam = SB_LINERIGHT;	// one line to the right.
						}
					}
				}
			}
			if (msg)		// Need to scroll?
			{
				ptAnchor = point;	// Save mouse position.
				do					// Repeat scrolling
				{
					m_pDragImage->DragLeave(this);							// Erase the drag image.
					::SendMessage(m_tvnFolders.m_hWnd, msg, wParam, NULL);	// Do it.
					m_pDragImage->DragEnter(this, point);					// Restore the drag image.
					GetCursorPos(&ptCurrent);		// Get mouse position.
					ScreenToClient(&ptCurrent);		// Convert to client coordinates
					// Check for pending mouse messages, but do not remove them from the message queue.
					if (PeekMessage(&message, NULL, WM_MOUSEFIRST, WM_MOUSELAST, PM_NOREMOVE))
					{
						// If left button is released or right button is pressed,
						// stop scrolling and let the message be processed.
						if (message.message == WM_LBUTTONUP || message.message == WM_RBUTTONDOWN)
							break;
					}
					else
					{
						if (m_intScrollRate)			// If scroll rate is set,
							Sleep(m_intScrollRate);		// waste some time before next scroll event.
					}
				// If requested, continue scrolling until the mouse moves.
				} while (ptAnchor == ptCurrent);
			}
		}
	}
	CFormView::OnMouseMove(nFlags, point);
}

/////////////////////////////////////
// CSUView "helper" functions

// Fills the tree control from the current document.
// It is first called with the path_offset parm set to the root folder,
// then calls itself recursively to process all folders.
// Returns FALSE (and thus halts recursion) on any error.
BOOL CSUView::Process_Path(UINT path_offset, HTREEITEM parent)
{
	CString title;
	CString * pName;
	HTREEITEM last_item;//, this_item;
	UINT child, i;
	TV_INSERTSTRUCT tvi;

	m_pMyDoc->m_pPathList[path_offset].processed = TRUE;		// Mark current folder as processed.
	title = *m_pMyDoc->m_pPathList[path_offset].path;			// Get path.
	m_pMyDoc->Friendly_Name(title);								// Get name of last folder in path.
	tvi.item.pszText = title.GetBuffer(title.GetLength());
	tvi.item.mask = TVIF_IMAGE | TVIF_SELECTEDIMAGE | TVIF_STATE | TVIF_TEXT | TVIF_PARAM;
	tvi.item.state = INDEXTOSTATEIMAGEMASK(m_pMyDoc->m_pPathList[path_offset].selected);
	tvi.item.stateMask = TVIS_STATEIMAGEMASK;
	tvi.item.iSelectedImage = tvi.item.iImage = (m_pMyDoc->m_pPathList[path_offset].locked ? 2 : 0);
	tvi.item.lParam = path_offset;
	tvi.hParent = parent;
	tvi.hInsertAfter = TVI_SORT;
	last_item = m_tvnFolders.InsertItem(&tvi);
	while(m_pMyDoc->Has_Children(path_offset, &child))	// Process any child folders.
	{
		if (!Process_Path(child, last_item))
			return FALSE;
	}
	// Now process any bookmarks in the current folder.
	for (i = 0; i < m_pMyDoc->m_intItemCount; i++)
	{
		if (m_pMyDoc->m_pDocElements[i].path_offset == path_offset)
		{
			pName = m_pMyDoc->m_pDocElements[i].page;
			tvi.item.pszText = pName->GetBuffer(pName->GetLength());
			tvi.item.mask = TVIF_IMAGE | TVIF_SELECTEDIMAGE | TVIF_STATE | TVIF_TEXT | TVIF_PARAM;
			tvi.item.state = 0;
			tvi.item.stateMask = TVIS_STATEIMAGEMASK;
			tvi.item.iImage = tvi.item.iSelectedImage = 1;
			tvi.item.lParam = i;
			tvi.hParent = last_item;
			tvi.hInsertAfter = TVI_SORT;
			m_tvnFolders.InsertItem(&tvi);
		}
	}
	return TRUE;
}

// Returns TRUE if the specified tree item is a folder.
BOOL CSUView::IsFolder(HTREEITEM hItem)
{
	TV_ITEM my_item;

	my_item.mask = TVIF_IMAGE;
	my_item.hItem = hItem;
	m_tvnFolders.GetItem(&my_item);
	return (my_item.iImage == 0 || my_item.iImage == 2);
}

// Returns TRUE if the specified tree item is a bookmark.
BOOL CSUView::IsBookmark(HTREEITEM hItem)
{
	TV_ITEM my_item;

	my_item.mask = TVIF_IMAGE;
	my_item.hItem = hItem;
	m_tvnFolders.GetItem(&my_item);
	return (my_item.iImage == 1);
}

// Called when: 
//    bRename = FALSE: User drops a bookmark on a valid location
//    bRename = TRUE:  User has renamed a bookmark.
// 
// Note that when bRename is TRUE, the OnEndlabeledit handler has
// already updated the document accordingly and set the m_hDragSource
// and m_hDragDest handles to the proper values. Thus, all we have to do
// is move the renamed item so it gets put into the tree in the proper sort order.
// This may seem complicated, but it's much faster than rebuilding the entire tree!
void CSUView::Move_Bookmark(BOOL bRename)
{
	HTREEITEM hNewItem, hTargetFolder;
	UINT folder_offset;
	TV_ITEM item2;
	TV_INSERTSTRUCT tvi;
	char buff1[MAX_PATH], buff2[MAX_PATH];

	if (m_hDragDest == NULL || m_hDragSource == NULL)
		return;

	// If we're not called as part of a rename operation,
	if (!bRename)
	{
		// Get handle of destination folder.
		if (IsFolder(m_hDragDest))
			hTargetFolder = m_hDragDest;
		else
			hTargetFolder = m_tvnFolders.GetParentItem(m_hDragDest);
		// If same as source folder, nothing to do.
		if (m_tvnFolders.GetParentItem(m_hDragSource) == m_hDragDest)
		{
			m_tvnFolders.SelectItem(m_hDragSource);
			return;
		}
	}
	else	// If we _are_ in a rename, handles have already been set.
	{
		hTargetFolder = m_hDragDest;
	}
	// Retrieve info for the bookmark being moved.
	tvi.item.mask = TVIF_TEXT | TVIF_IMAGE | TVIF_SELECTEDIMAGE | TVIF_CHILDREN | TVIF_STATE | TVIF_PARAM;
	item2.mask = TVIF_TEXT | TVIF_IMAGE;
	tvi.item.hItem = m_hDragSource;
	tvi.item.pszText = buff1;
	tvi.item.cchTextMax = sizeof(buff1);
	item2.pszText = buff2;
	item2.cchTextMax = sizeof(buff2);
	m_tvnFolders.GetItem(&tvi.item);
	tvi.item.iSelectedImage = tvi.item.iImage;
	// If we're not in a rename operation,
	if (!bRename)
	{
		// Scan parent folder for a bookmark with the same name.
		hNewItem = m_tvnFolders.GetChildItem(hTargetFolder);
		while(hNewItem != NULL)
		{
			item2.hItem = hNewItem;
			m_tvnFolders.GetItem(&item2);
			if (item2.iImage == 1)
			{
				// Give error message and bail if dupe found.
				if (stricmp(tvi.item.pszText, item2.pszText)==0)
				{
					m_tvnFolders.EnsureVisible(hNewItem);
					AfxMessageBox("Target folder already contains a\n"
						"bookmark with the same name.", MB_ICONINFORMATION);
					m_tvnFolders.SelectItem(m_hDragSource);
					return;
				}
			}
			hNewItem = m_tvnFolders.GetNextSiblingItem(hNewItem);
		}
		// Retrieve path offset for destination folder.
		folder_offset = m_tvnFolders.GetItemData(hTargetFolder);
		// Update document with source item's new parent folder.
		m_pMyDoc->m_pDocElements[/*item_offset*/ tvi.item.lParam].path_offset = folder_offset;
		// Mark document as dirty.
		m_pMyDoc->SetModifiedFlag();
	}
	// Insert the moved item into the tree.
	tvi.hParent = hTargetFolder;
	tvi.hInsertAfter = TVI_SORT;
	hNewItem = m_tvnFolders.InsertItem(&tvi);
	// Make sure new item is visible.
	m_tvnFolders.EnsureVisible(hNewItem);
	// Select the item.
	m_tvnFolders.SelectItem(hNewItem);
	// Delete the original item.
	m_tvnFolders.DeleteItem(m_hDragSource);
	// Reset handles for drag/drop source and destination.
	m_hDragSource = NULL;
	m_hDragDest = NULL;
}

// This recursive function adjusts the bookmark offsets
// after a bookmark has been deleted from the document.
// It is first called with the handle of the root folder.
// It then calls itself with the handle of any subfolders
// found as it walks the current folder.
void CSUView::Adjust_Bookmarks(HTREEITEM parent)
{
	HTREEITEM hCurItem;
	UINT offset;

	hCurItem = m_tvnFolders.GetChildItem(parent);
	while (hCurItem != NULL)
	{
		if (IsFolder(hCurItem))
			Adjust_Bookmarks(hCurItem);
		else
		{
			offset = m_tvnFolders.GetItemData(hCurItem);
			if (offset >= m_nDeletedItem)
			{
				--offset;
				m_tvnFolders.SetItemData(hCurItem, offset);
			}
		}
		hCurItem = m_tvnFolders.GetNextSiblingItem(hCurItem);
	}
}

// Tells whether or not the specified tree item can be moved.
BOOL CSUView::IsLocked(HTREEITEM item)
{
	TV_ITEM my_item;

	my_item.mask = TVIF_IMAGE;
	my_item.hItem = item;
	m_tvnFolders.GetItem(&my_item);
	return (my_item.iImage >= 2);
}

// Handles folder move.
//    bRename = FALSE: User has dragged a folder to a new location.
//    bRename = TRUE: User has successfully renamed a folder.
//
// Note that when bRename = TRUE, the OnEndlabeledit handler has already
// taken care of updating the document and set the m_hDragSource and
// m_hDragDest member variables appropriately. Thus, this function simply
// moves the renamed folder so that is placed in the tree with the proper sort order.
void CSUView::Move_Folder(BOOL bRename)
{
	HTREEITEM hTargetFolder, hScratch, hNewItem;
	UINT source_offset, dest_offset;
	TV_ITEM info;

	if (m_hDragSource == NULL || m_hDragDest == NULL)
		return;
	// Unless move is part of a rename,
	if (!bRename)
	{
		// If drag destination was a bookmark,
		// get its parent folder.
		if (IsFolder(m_hDragDest))
			hTargetFolder = m_hDragDest;
		else
			hTargetFolder = m_tvnFolders.GetParentItem(m_hDragDest);
		// Nothing to do if folder was dropped on itself.
		if (m_hDragSource == hTargetFolder)
			return;
		// Nothing to do if folder was dropped on its own parent folder.
		hScratch = m_tvnFolders.GetParentItem(m_hDragSource);
		if (hScratch == hTargetFolder)
		{
			m_tvnFolders.SelectDropTarget(NULL);
			m_tvnFolders.SelectItem(m_hDragSource);
			return;
		}
		hScratch = m_tvnFolders.GetParentItem(hTargetFolder);
		// Make sure folder wasn't dropped on one of its own children.
		while (hScratch != NULL)
		{
			if (hScratch != m_hDragSource)
				hScratch = m_tvnFolders.GetParentItem(hScratch);
			else
			{
				AfxMessageBox("Can't move folder to one of its own subfolders!", MB_ICONINFORMATION);
				return;
			}
		}
		// Retrieve offsets for the source and destination folders.
		source_offset = m_tvnFolders.GetItemData(m_hDragSource);
		dest_offset = m_tvnFolders.GetItemData(hTargetFolder);
		// Tell document to adjust its path information, reporting error if it fails.
		if (!m_pMyDoc->Move_Folder(source_offset, dest_offset))
		{
			AfxMessageBox("Invalid folder destination.\n\n"
				"Either the resulting path is too long\n"
				"or a folder with the same name already exists.", MB_ICONINFORMATION);
			return;
		}
		// Mark document as dirty.
		m_pMyDoc->SetModifiedFlag();
	}
	else
	{
		hTargetFolder = m_hDragDest;
	}
	// Get handle to source folder's parent. (Note: this handle will
	// never be NULL because the UI won't allow the root folder to be moved).
	hScratch = m_tvnFolders.GetParentItem(m_hDragSource);
	// Move the source folder to the destination folder in the tree,
	// and save handle to newly created folder.
	hNewItem = Move_Tree_Folder(m_hDragSource, hTargetFolder);
	m_tvnFolders.EnsureVisible(hNewItem);
	m_tvnFolders.SelectItem(hNewItem);
	// Delete the (now empty) source folder.
	if (!bRename)
	{
		// Get path offset for source folder's parent path.
		info.mask = TVIF_PARAM;
		info.hItem = hScratch;
		m_tvnFolders.GetItem(&info);
		// Retrieve source folder's parent's selection state
		// (This was set appropriately by CSUDoc::Move_Folder)
		info.state = (m_pMyDoc->m_pPathList[info.lParam].selected) << 12;
		info.stateMask = TVIS_STATEIMAGEMASK;
		info.mask = TVIF_STATE;
		// Apply the proper selection state.
		m_tvnFolders.SetItem(&info);
		// Adjust selection state for relevant parts of the tree.
		Set_State_Up(hNewItem);
		Set_State_Up(hScratch);
	}
	m_tvnFolders.DeleteItem(m_hDragSource);
	// Make sure the moved folder is visible and selected.
	m_hDragSource = NULL;
	m_hDragDest = NULL;
}

// Moves a folder item and all of its children
// to the specified destination folder.
// Returns handle to the newly created folder item.
HTREEITEM CSUView::Move_Tree_Folder(HTREEITEM hSource, HTREEITEM hDest)
{
	TV_INSERTSTRUCT		tvi;
	TCHAR				buff[_MAX_PATH];
	HTREEITEM			hNewItem, hFirstChild;

	// Retrieve information about source item.
	tvi.item.hItem = hSource;
	tvi.item.cchTextMax = sizeof(buff);
	tvi.item.pszText = buff;
	tvi.item.mask = TVIF_CHILDREN | TVIF_HANDLE | TVIF_IMAGE | TVIF_STATE | TVIF_TEXT | TVIF_PARAM;
	tvi.item.stateMask = TVIS_STATEIMAGEMASK;
	m_tvnFolders.GetItem(&tvi.item);
	// Insert a copy of the item into the target.
	// Note: Caller is responsible for deleting the original source item.
	tvi.hParent = hDest;
	// Place item in proper sorted position.
	tvi.hInsertAfter = TVI_SORT;
	tvi.item.mask = TVIF_IMAGE | TVIF_STATE | TVIF_TEXT | TVIF_PARAM;
	hNewItem = m_tvnFolders.InsertItem(&tvi);

	// Handle any child items.
	while ((hFirstChild = m_tvnFolders.GetChildItem(hSource)) != NULL)
	{
		Move_Tree_Folder(hFirstChild, hNewItem);    // recursively transfer all the items
		m_tvnFolders.DeleteItem(hFirstChild);		// delete the first child and all its children
	}
	// Return handle to the moved item.
	return hNewItem;
}

// Called recursively after user moves a folder.
// Ensures that the folder he just moved is displayed.
BOOL CSUView::Locate_Folder(HTREEITEM parent, UINT offset)
{
	HTREEITEM cur_item;

	if (offset == (m_tvnFolders.GetItemData(parent)))	// If item offset matches path offset,
	{

		m_tvnFolders.EnsureVisible(parent);		// Make sure tree is opened to the item.
		m_tvnFolders.SelectItem(parent);		// Select the item.
		m_hCurItem = parent;					// Save current item.
		Set_State_Up(parent);					// Adjust selection state for parents.
		return TRUE;							// We're done.
	}
	if (!m_tvnFolders.ItemHasChildren(parent))	// Keep looking if item has no children.
		return FALSE;
	cur_item = m_tvnFolders.GetChildItem(parent);	// Get first child item.
	while (cur_item != NULL)						// Loop through remaining children.
	{
		if (IsFolder(cur_item))						// If item is a folder,
			if (Locate_Folder(cur_item, offset))	// Call ourselves.
				return TRUE;						// If that was the one, we're done.
		cur_item = m_tvnFolders.GetNextSiblingItem(cur_item);	// Otherwise, move on to the next item.
	}
	return FALSE;
}

// Called recursively when a tree item's selection state changes.
// Sets or resets the selection state of specified item
// and all of its child folders.
void CSUView::Set_State_Down(HTREEITEM item, BOOL state)
{
	HTREEITEM hCurItem;
	TV_ITEM item_info;
	UINT item_state;

	if (state)						// Convert from TRUE/FALSE to
		item_state = 2 << 12;		// appropriate state mask value.
	else
		item_state = 1 << 12;
	item_info.mask = TVIF_STATE;
	item_info.hItem = item;
	item_info.state = item_state;
	item_info.stateMask = TVIS_STATEIMAGEMASK;
	m_tvnFolders.SetItem(&item_info);			// Set current item state.
	item_info.mask = TVIF_PARAM | TVIF_CHILDREN;	// Retrieve path offset and child flag.
	m_tvnFolders.GetItem(&item_info);
	m_pMyDoc->m_pPathList[item_info.lParam].selected = (state ? 2 : 1);	// Update selection state in the document.
	if (item_info.cChildren)						// If item has children,
	{
		hCurItem = m_tvnFolders.GetChildItem(item);	// Get first child.
		while (hCurItem != NULL)
		{
			item_info.mask = TVIF_IMAGE;			// Get item info.
			item_info.hItem = hCurItem;
			m_tvnFolders.GetItem(&item_info);
			if (item_info.iImage != 1)				// If it's a folder,
				Set_State_Down(hCurItem, state);	// process it.
			hCurItem = m_tvnFolders.GetNextSiblingItem(hCurItem);	// Next item.
		}
	}
}

// Called recursively when a tree item's selection state changes.
// Adjusts selection state of item's parents.
void CSUView::Set_State_Up(HTREEITEM item)
{
	HTREEITEM hCurItem, hParentItem;
	BOOL bAll, bNone;
	TV_ITEM tvi;
	UINT nItemState;
	char szBuff[_MAX_PATH];

	hParentItem = m_tvnFolders.GetParentItem(item);	// Get item's parent.
	if (hParentItem == NULL)							// Done if item is the root.
		return;
	bAll = TRUE;									// Set flag.
	bNone = TRUE;
	hCurItem = m_tvnFolders.GetChildItem(hParentItem);	// Walk through item's children.
	tvi.stateMask = TVIS_STATEIMAGEMASK;
	tvi.cchTextMax = MAX_PATH;
	tvi.pszText = szBuff;
	while (hCurItem != NULL)
	{
		tvi.mask = TVIF_IMAGE | TVIF_STATE | TVIF_TEXT;	// Retrieve item type & selection state.
		tvi.hItem = hCurItem;
		m_tvnFolders.GetItem(&tvi);
		if (tvi.iImage != 1)					// Nothing to do if item is a bookmark.
		{
			if ((tvi.state & TVIS_STATEIMAGEMASK) != 0x2000)					// If item is not fully selected,
				bAll = FALSE;						// Clear flag and stop looking.
			if ((tvi.state & TVIS_STATEIMAGEMASK)!= 0x1000)
				bNone = FALSE;
		}
		hCurItem = m_tvnFolders.GetNextSiblingItem(hCurItem);	// Next item.
	}
	tvi.mask = TVIF_PARAM | TVIF_STATE | TVIF_TEXT;			// Get item's path offset.
	tvi.hItem = hParentItem;
	m_tvnFolders.GetItem(&tvi);
	if (bNone)
	{
		if ((tvi.state & TVIS_STATEIMAGEMASK) != 0x1000)
		{
			nItemState = 0x3000;
			m_pMyDoc->m_pPathList[tvi.lParam].selected = 3;
		}
		else
			nItemState = tvi.state;
	}
	else
	{
		if (bAll)	// If all children were selected,
					// then item is fully selected
			nItemState = (m_pMyDoc->m_pPathList[tvi.lParam].selected = 2) << 12;
		else	// Otherwise,
				// item is partially selected.
			nItemState = (m_pMyDoc->m_pPathList[tvi.lParam].selected = 3) << 12;
	}
	// Set selection state in the tree.
	tvi.mask = TVIF_STATE;
	tvi.state = nItemState;
	tvi.stateMask = TVIS_STATEIMAGEMASK;
	m_tvnFolders.SetItem(&tvi);
	// Proceed with next parent.
	Set_State_Up(hParentItem);
}

// Adjusts folder path offsets after a folder has been removed.
void CSUView::Adjust_Folder_Offsets(HTREEITEM item, UINT offset)
{
	TV_ITEM info;
	HTREEITEM hChildItem, hCurItem;

	hCurItem = item;
	while (hCurItem != NULL)
	{
		info.mask = TVIF_CHILDREN | TVIF_IMAGE | TVIF_PARAM;
		info.hItem = hCurItem;
		m_tvnFolders.GetItem(&info);
		if (info.iImage != 1)
		{
			if (info.cChildren != 0)
			{
				hChildItem = m_tvnFolders.GetChildItem(hCurItem);
				ASSERT(hChildItem != NULL);
				Adjust_Folder_Offsets(hChildItem, offset);
			}
			if ((UINT) (info.lParam) >= offset)
			{
				--info.lParam;
				info.mask = TVIF_PARAM;
				m_tvnFolders.SetItem(&info);
			}
		}
		hCurItem = m_tvnFolders.GetNextSiblingItem(hCurItem);
	}
}

// Searches document for duplicate URLs and lets user choose which ones to keep.
BOOL CSUView::Weed_Duplicates()
{
	DUP_URL_ELEMENT* pDupeItems = NULL;
	DUP_URL_SET* pDupeSets = NULL;
	void *pTemp;
	UINT nMaxSets = 128, nSetCount = 0;
	UINT i, j, start, nCount, end;
	CWeedDlg dlg;
	CStatDlg dlgStat;
	BOOL found = FALSE, bail = FALSE;
	CString msg;

	nCount = m_pMyDoc->m_intItemCount;			// Get number of bookmarks.
	pDupeItems = new DUP_URL_ELEMENT[nCount];		// Create array to hold info.
	pDupeSets = new DUP_URL_SET[nMaxSets];		// Create array to hold more infor.
	if (pDupeItems == NULL || pDupeSets == NULL)		// Bail if unable to create arrays.
	{
		AfxMessageBox("Insufficient memory to perform this operation", MB_ICONEXCLAMATION);
		return FALSE;
	}
	dlgStat.CenterWindow();		// Let the user know what's going on.
	dlgStat.Show_Message("Building working URL list...");
	// Load the info array from the document.
	for (i = 0; i < nCount; i++)
	{
		pDupeItems[i].url = m_pMyDoc->m_pDocElements[i].url;
		pDupeItems[i].offset = i;
		pDupeItems[i].remove = FALSE;
	}
	// Sort the array by URL.
	dlgStat.Show_Message("Sorting...");
	qsort((void*) pDupeItems, nCount, sizeof(DUP_URL_ELEMENT), Compare_Dupe_URLs);
	dlgStat.Show_Message("Searching...");
	// Walk the array and check for duplicates.
	for (i = 0; i < nCount - 1; i++)
	{
		start = 0;		// Initialize offsets for current set of dupes.
		end = 0;
		// Consecutive URLs match?
		if (pDupeItems[i].url->Compare(*pDupeItems[i+1].url) == 0)
		{
			found = TRUE;		// Found a dupe.
			start = i;			// Save offsets for this set of dupes.
			end = i + 1;
			while (end < nCount -1)		// Loop through rest of array.
			{
				// Does next URL match?
				if (pDupeItems[start].url->Compare(*pDupeItems[end+1].url) == 0)
					++end;	// Update end offset if so.
				else
					break;	// Done searching if not.
			}
			// See if Dupe set array needs to be bigger.
			if (nSetCount > nMaxSets)
			{
				// Reallocate it if so.
				pTemp = realloc(pDupeSets, nSetCount * sizeof(DUP_URL_SET));
				if (pTemp == NULL)
				{
					// If reallocation fails, report error and quit.
					AfxMessageBox("Insufficient memory to perform this operation", MB_ICONEXCLAMATION);
					bail = TRUE;
					found = FALSE;
					break;
				}
				else
				{
					// Otherwise, set new array pointer.
					pDupeSets = (DUP_URL_SET*) pTemp;
					nMaxSets = nSetCount;
				}
			}
			// Save data for current set of dupes.
			pDupeSets[nSetCount].start = start;
			pDupeSets[nSetCount].end = end;
			++nSetCount;
		}
		if (start)		// If dupe was found,
			i = end;	// Set loop counter past last dupe.
	}
	// Hide the status display for now.
	dlgStat.ShowWindow(SW_HIDE);
	// If dupes were found and no errors occurred,
	if (found && (!bail))
	{
		dlg.m_pimgIcons = &m_imgTree;		// Give dialog pointers to our
		dlg.m_pimgBoxes = &m_imgState;		// image lists,
		dlg.m_pDupeItems = pDupeItems;		// duplicate array,
		dlg.m_pDupeSets = pDupeSets;		// duplicate set array,
		dlg.m_nSetCount = nSetCount;		// # of sets of dupes.
		dlg.m_pItemList = m_pMyDoc->m_pDocElements;	// Pointer to document data
		dlg.m_pPathList = m_pMyDoc->m_pPathList;	// and path data.
		bail = (dlg.DoModal() != IDOK);		// Launch dialog, bail if user cancels.
	}
	if (bail)		// Give bail out message.
	{
		AfxMessageBox("Operation cancelled\nNo bookmarks were removed.", MB_ICONINFORMATION);
		found = FALSE;
	}
	else
	{
		if (found)		// Dupes found?
		{
			if (dlg.m_nRemoveCount == 0)	// Any selected for removal?
			{
				AfxMessageBox("No duplicates were marked for removal");
				found = FALSE;
			}
			else
			{
				// Give user one last chance to bail.
				msg.Format("%u bookmarks are marked for removal\n\nProceed?", dlg.m_nRemoveCount);
				if (AfxMessageBox(msg, MB_ICONQUESTION | MB_YESNO) != IDYES)
				{
					AfxMessageBox("Operation cancelled", MB_ICONINFORMATION);
					found = FALSE;
				}
				else
				{
					// Let user know we're working on it.
					dlgStat.Show_Message("Removing marked items...");
					// Remove the marked items from the document.
					for (i = 0; i < nCount; i++)
					{
						if (pDupeItems[i].remove)
						{
							m_pMyDoc->Delete_Item(pDupeItems[i].offset);
							for (j = i + 1; j < nCount; j++)
							{
								if (pDupeItems[j].offset >= pDupeItems[i].offset)
									--pDupeItems[j].offset;
							}
						}
					}
				}
			}
		}
		else		// Let user know if no dupes exist.
			AfxMessageBox("No duplicate URLs found.", MB_ICONINFORMATION);
	}
	// Clean up.
	delete [] pDupeItems;
	delete [] pDupeSets;
	dlgStat.DestroyWindow();
	return found;
}

// Called recursively to expand or collapse all folders in the tree.
// Parms:
//    hItem = current item to process (NULL on first call, indicating root folder)
//    expand = flag; TRUE to expand, FALSE to collapse.
void CSUView::Expand_All(HTREEITEM hItem, BOOL expand)
{
	TV_ITEM info;
	HTREEITEM hCurItem, hThisItem;
	UINT cmd;
	BOOL root;

	cmd = expand ? TVE_EXPAND : TVE_COLLAPSE;		// Set desired command constant.
	if (hItem == NULL)								// Root folder?
	{
		hThisItem = m_tvnFolders.GetRootItem();		// Get its handle
		root = TRUE;								// Set flag.
	}
	else
	{
		hThisItem = hItem;							// Use provided handle
		root = FALSE;								// Clear flag.
	}
	info.mask = TVIF_CHILDREN | TVIF_IMAGE;			// Retrieve image and child info.
	info.hItem = hThisItem;
	m_tvnFolders.GetItem(&info);
	if (info.cChildren)									// Item has children?
	{
		hCurItem = m_tvnFolders.GetChildItem(hThisItem);	// Get first child.
		do
		{
			info.hItem = hCurItem;							// Get its info.
			m_tvnFolders.GetItem(&info);
			if (info.iImage != 1)							// Is it a folder?
				Expand_All(hCurItem, expand);				// Process it.
			hCurItem = m_tvnFolders.GetNextSiblingItem(hCurItem);	// Next item.
		}
		while (hCurItem != NULL);
	}
	if (root)											// Root folder is always expanded.
	{
		m_tvnFolders.Expand(hThisItem, TVE_EXPAND);
		if (expand)
		{
			hCurItem = m_tvnFolders.GetSelectedItem();
			if (hCurItem != NULL)
				m_tvnFolders.EnsureVisible(hCurItem);
			else
			{
				m_tvnFolders.SelectItem(hThisItem);
				m_tvnFolders.EnsureVisible(hThisItem);
			}
		}
		else
			m_tvnFolders.SelectItem(hThisItem);
	}
	else
		m_tvnFolders.Expand(hThisItem, cmd);
}

// Searches the tree, starting at the specified item,
// looking for a bookmark that matches the current search criteria.
// Called by OnEditFind (which sets the criteria) and
// OnEditFindNext (which continues the search with existing criteria).
// If matching bookmark is found, item is selected in the tree
// and function returns TRUE.
// Otherwise, existing selection remains unchanged and function returns FALSE.
BOOL CSUView::Search_Tree(HTREEITEM hStart)
{
	HTREEITEM hResult = NULL, hCurItem;
	TV_ITEM info;
	UINT offset, nTestFunc;
	BOOL result;
	CString strTarget(m_strSearchText);
	CString strTest;

	strTarget.MakeLower();
	info.mask = TVIF_IMAGE | TVIF_PARAM;		// Set mask to retrieve iImage and lParam
	// Determine which test function to call
	nTestFunc = (m_intSearchIn * 2) + (m_bSearchCase ? 1 : 0);
	hCurItem = hStart;
	while (hCurItem != NULL)					// Search until no items left.
	{
		info.hItem = hCurItem;					// Set handle to desired item.
		m_tvnFolders.GetItem(&info);			// Retrieve item data.
		if (info.iImage == 1)					// If item is a bookmark,
		{
			offset = (UINT) info.lParam;		// Force item offset to a UINT.
			switch (nTestFunc)					// Call approprate test function.
			{
			case 0:								// Page and URL, case-sensitive.
				if (m_pMyDoc->m_pDocElements[offset].page->Find(m_strSearchText) != -1)
					result = TRUE;
				else
					result = m_pMyDoc->m_pDocElements[offset].url->Find(m_strSearchText) != -1;
				break;
			case 1:								// Page and URL, case-insensitive.
				strTest = *m_pMyDoc->m_pDocElements[offset].page;
				strTest.MakeLower();
				if (strTest.Find(strTarget) != -1)
					result = TRUE;
				else
				{
					strTest = *m_pMyDoc->m_pDocElements[offset].url;
					strTest.MakeLower();
					result = strTest.Find(strTarget) != -1;
				}
				break;
			case 2:								// Page only, case-sensitive.
				result = m_pMyDoc->m_pDocElements[offset].page->Find(m_strSearchText) != -1;
				break;
			case 3:								// Page only, case-insensitive.
				strTest = *m_pMyDoc->m_pDocElements[offset].page;
				strTest.MakeLower();
				result = strTest.Find(strTarget) != -1;
				break;
			case 4:								// URL only, case-sensitive.
				result = m_pMyDoc->m_pDocElements[offset].url->Find(m_strSearchText) != -1;
				break;
			case 5:								// URL only, case-insensitive.
				strTest = *m_pMyDoc->m_pDocElements[offset].url;
				strTest.MakeLower();
				result = strTest.Find(strTarget) != -1;
			}
			if (result)							// If item matched,
			{
				m_tvnFolders.EnsureVisible(hCurItem);	// Make sure it's visible.
				m_tvnFolders.SelectItem(hCurItem);		// Select it.
				return TRUE;							// Indicate success.
			}
		}
		hCurItem = Next_Tree_Item(hCurItem);			// Continue with next item.
	}
	return FALSE;		// Item not found, so return FALSE.
}

// Called by Search_Tree()
// Used to walk through the tree in top-down order,
// as if the entire tree were expanded.
// Input: Handle to current item.
// Returns: Handle to next item, NULL if there isn't one.
HTREEITEM CSUView::Next_Tree_Item(HTREEITEM hItem)
{
	HTREEITEM hNextItem = NULL, hCurItem = hItem;
	hNextItem = m_tvnFolders.GetNextItem(hCurItem, TVGN_CHILD);		// Does item have any children?
	if (hNextItem != NULL)											// Return first child if so.
		return hNextItem;
	hNextItem = m_tvnFolders.GetNextItem(hCurItem, TVGN_NEXT);		// Any siblings left?
	if (hNextItem != NULL)											// Return sibling if so.
		return hNextItem;
	do
	{
		hCurItem = m_tvnFolders.GetNextItem(hCurItem, TVGN_PARENT);	// Get parent.
		if (hCurItem == NULL)										// Done if item has no parent.
			return NULL;
		else
			hNextItem = m_tvnFolders.GetNextItem(hCurItem, TVGN_NEXT);	// Get parent's next sibling.
	} while (hNextItem == NULL);	// Walk back up the tree until we either reach the root
	                                // or find a parent item that still has siblings.
	return hNextItem;
}

// Create new Netscape BOOKMARK.HTM file from the current document.
// Parm bBackup determines whether or not backup copy will be created.
// If not specified, defaults to TRUE.
void CSUView::Write_NS_File(BOOL bBackup)
{
	CStdioFile file;
	CFileStatus status;
	CString line, file_name, bak_name;
	BOOL renamed = FALSE;
	int pos;

	file_name = m_pMyDoc->m_strNSPath;
	if (bBackup)
	{
		pos = file_name.ReverseFind('.');
		bak_name = file_name.Left(pos);
		bak_name += ".bak";
		status.m_ctime = 0;
		status.m_size = 0;
		CFile::GetStatus(bak_name, status);
		if (status.m_ctime == 0 && status.m_size == 0)
		{
			try {
				CFile::Rename(file_name, bak_name);
			}
			catch ( CFileException* e) {
				e->Delete();
				AfxMessageBox("Could not create backup copy.\nOperation aborted.", MB_ICONEXCLAMATION);
				return;
			}
			renamed = TRUE;
		}
		else
		{
			if (AfxMessageBox("Backup copy already exists.\n\nReplace it?", MB_YESNO | MB_ICONQUESTION) == IDYES)
			{
				try {
					CFile::Remove(bak_name);
				}
				catch (CFileException* e) {
					e->Delete();
					AfxMessageBox("Could not delete existing backup.\nOperation aborted.", MB_ICONEXCLAMATION);
					return;
				}
				try {
					CFile::Rename(file_name, bak_name);
				}
				catch (CFileException* e) {
					e->Delete();
					AfxMessageBox("Could not create backup copy.\nOperation aborted.", MB_ICONEXCLAMATION);
					return;
				}
				renamed = TRUE;
			}
		}
	}
	if (!file.Open(file_name, CFile::modeCreate | CFile::modeWrite, NULL))
	{
		line.Format("Could not create or replace %s", file_name);
		AfxMessageBox(line, MB_ICONINFORMATION);
		if (bBackup)
		{
			if (renamed)
			{
				try	{
					CFile::Rename(bak_name, file_name);
				}
				catch (CFileException* e) {
					e->Delete();
					AfxMessageBox("Could not restore from backup.", MB_ICONEXCLAMATION);
					return;
				}
				AfxMessageBox("File restored from backup copy.", MB_ICONINFORMATION);
			}
		}
		return;
	}
	m_pFile = &file;
	Write_NS_Folder(m_tvnFolders.GetRootItem());
	file.Close();
	m_pFile = NULL;
}

// Subroutine recurses through the tree
// and appends each folder's contents to BOOKMARK.HTM.
// Note that even though the file is opened in text mode,
// each line written is terminated with "\r\n", which actually puts
// a CR/CR/LF sequence at the end of each line. This seems bizarre, but
// that's actually the way Netscape writes the file!
void CSUView::Write_NS_Folder(HTREEITEM hThisFolder)
{
	static CString line;
	static int indent = 0;
	static char szNameBuff[MAX_PATH];
	char *buff;
	HTREEITEM hThisItem;
	TV_ITEM info;
	UINT nItemOffset;


	info.mask = TVIF_IMAGE | TVIF_PARAM | TVIF_STATE | TVIF_TEXT;
	info.stateMask = TVIS_STATEIMAGEMASK;
	info.pszText = szNameBuff;
	info.cchTextMax = sizeof(szNameBuff);

	// If this is the root directory,
	if (hThisFolder == m_tvnFolders.GetRootItem())
	{
		// Write the file header.
		line = "<!DOCTYPE NETSCAPE-Bookmark-file-1>\r\n"
				"<!-- This is an automatically generated file.\r\n"
				"It will be read and overwritten.\r\n"
				"Do Not Edit! -->\r\n";
		m_pFile->WriteString(line);
		line.Format("<TITLE>%s</TITLE>\r\n<H1>%s</H1>\r\n", m_pMyDoc->m_strNSPageTitle, m_pMyDoc->m_strNSPageTitle);
		m_pFile->WriteString(line);
	}
	info.hItem = hThisFolder;
	m_tvnFolders.GetItem(&info);
	// Done if folder is not selected.
	if ((info.state >> 12) < 2)
		return;
	// Handle Netscape's indentions.
	buff = new char[indent + 5];
	memset(buff, 32, indent + 4);
	buff[indent] = '\0';
	// If not the root folder, write the HTML tag for the folder name.
	if (hThisFolder != m_tvnFolders.GetRootItem())
	{
		line.Format("%s<DT><H3 FOLDED ADD_DATE=\"0\">%s</H3>\r\n", buff, szNameBuff);
		m_pFile->WriteString(line);
	}
	// Write the HTML tag that marks the start of a folder's contents.
	line.Format("%s<DL><p>\r\n", buff);
	m_pFile->WriteString(line);
	// Adjust the indent.
	buff[indent] = ' ';
	indent += 4;
	buff[indent] = '\0';
	hThisItem = m_tvnFolders.GetChildItem(hThisFolder);
	while (hThisItem != NULL)
	{
		info.hItem = hThisItem;
		m_tvnFolders.GetItem(&info);
		if (info.iImage != 1)
		{
			if ((info.state >> 12) > 1)
				Write_NS_Folder(hThisItem);
		}
		else
		{
			nItemOffset = (UINT) info.lParam;
			line.Format("%s<DT><A HREF=\"%s\" ADD_DATE=\"%u\" LAST_MODIFIED=\"%u\" LAST_VISIT=\"%u\">%s</A>\r\n",
						buff,
						*m_pMyDoc->m_pDocElements[nItemOffset].url,
						m_pMyDoc->m_pDocElements[nItemOffset].add_date,
						m_pMyDoc->m_pDocElements[nItemOffset].mod_date,
						m_pMyDoc->m_pDocElements[nItemOffset].visit_date,
						szNameBuff);
			m_pFile->WriteString(line);
			// If item has a comment,
			if (m_pMyDoc->m_pDocElements[nItemOffset].comment != NULL)
			{
				// Write the HTML for the comment to the file.
				// Note: Comment lines are taken from the file verbatim.
				// They are not indented, and the string itself contains any "extra"
				// CR's that were present in the original file.
				// For the record, Netscape does not even handle its own comments
				// gracefully when they contain any line breaks!
				line.Format("<DD>%s\n", *m_pMyDoc->m_pDocElements[nItemOffset].comment);
				m_pFile->WriteString(line);
			}
		}
		hThisItem = m_tvnFolders.GetNextSiblingItem(hThisItem);
	}
	// Outdent after last folder item.
	indent -= 4;
	buff[indent] = '\0';
	// Write HTML tag indicating end of folder.
	line.Format("%s</DL><p>\r\n", buff);
	m_pFile->WriteString(line);
	// Delete the buffer used for indents.
	delete [] buff;
}


// Formats a date value as a string.
// Input: Date value to be formatted
//        Reference to a CString object to contain the formatted date.
// If date is 0, string will contain "???".
// Otherwise, it will contain the "long" date formatted according to the
// system settings (via Control Panel | Regional Settings | Date tab). 
void CSUView::Format_Date_Field(time_t nDate, CString& strDate)
{
	if (nDate == 0)
		strDate = "???";
	else
	{
		CTime time(nDate);
		strDate = time.Format("%#c");
	}
}

// QSort callback function used by Weed_Duplicates()
int __cdecl Compare_Dupe_URLs(const void * p1, const void * p2)
{
	ASSERT(p1 != NULL && p2 != NULL);

	const DUP_URL_ELEMENT* i1 = (DUP_URL_ELEMENT*) p1;
	const DUP_URL_ELEMENT* i2 = (DUP_URL_ELEMENT*) p2;

	return (i1->url->Compare(*i2->url));
}
