// WeedDlg.cpp : implementation file
//
//
// SyncURLs version 1.0
// Copyright 1998 Ziff-Davis Inc.
// First published in PC Magazine, US Edition
// January 6, 1998
// Written by Gregory A. Wolking
//

#include "stdafx.h"
#include "SyncURLs.h"
#include "MyList.h"
#include "WeedDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CWeedDlg dialog


CWeedDlg::CWeedDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CWeedDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CWeedDlg)
	//}}AFX_DATA_INIT
}


void CWeedDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CWeedDlg)
	DDX_Control(pDX, IDC_lstDupes, m_lstDupes);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CWeedDlg, CDialog)
	//{{AFX_MSG_MAP(CWeedDlg)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CWeedDlg message handlers

BOOL CWeedDlg::OnInitDialog() 
{
	UINT i, j, path_offset;
	int iItem = 0, nColumnWidth;
	CString strLine;
	LV_ITEM lvi;
	char buff[_MAX_PATH];
	CRect rect;

	CDialog::OnInitDialog();

	m_lstDupes.SetImageList(m_pimgIcons, LVSIL_SMALL);				// Attach image lists to the
	m_lstDupes.SetImageList(m_pimgBoxes, LVSIL_STATE);				// List control.
	// Create list control's columns.
	m_lstDupes.InsertColumn(0, "URL / Folder Name", LVCFMT_LEFT, -1, -1);
	m_lstDupes.InsertColumn(1, "Page Name", LVCFMT_LEFT, -1, 1);
	// Now that list control is set up, fill it.
	// For each set of dupes:
	for (i = 0; i < m_nSetCount; i++)
	{
		// Copy URL to buffer.
		strcpy(buff, *m_pDupeItems[m_pDupeSets[i].start].url);
		// Set structure elements for URL item.
		lvi.mask = LVIF_TEXT | LVIF_IMAGE | LVIF_STATE | LVIF_PARAM;
		lvi.iItem = iItem;						// Item offset.
		lvi.iSubItem = 0;						// No sub-item.
		lvi.state = (5 << 12);					// Displays "URL" instead of a checkbox.
		lvi.stateMask = LVIS_STATEIMAGEMASK;
		lvi.iImage = 3;							// Icon is a dash.
		lvi.pszText = buff;						// Pointer to URL text.
		lvi.cchTextMax = 0;
		lvi.lParam = -1;						// -1 indicates item is a header.
		m_lstDupes.InsertItem(&lvi);			// Add the item.
		++iItem;								// Next item index.
		lvi.state = (2 << 12);					// Blank checkbox for bookmarks.
		lvi.iImage = 1;							// Icon for bookmarks.
		// For each item in the current set of dupes:
		for (j = m_pDupeSets[i].start; j <= m_pDupeSets[i].end; j++)
		{
			// Copy folder path to buffer.
			path_offset = m_pItemList[m_pDupeItems[j].offset].path_offset;
			if (path_offset == 0)
				strLine = "Root Folder\\";
			else
				strLine = *m_pPathList[path_offset].path;
			strcpy(buff, strLine);
			// Set structure elements.
			lvi.mask = LVIF_TEXT | LVIF_IMAGE | LVIF_STATE | LVIF_PARAM;
			lvi.pszText = buff;
			lvi.iSubItem = 0;			// Puts item in Column 0.
			lvi.iItem = iItem;
			lvi.lParam = j;				// Offset to item in dupe array.
			m_lstDupes.InsertItem(&lvi);
			// Copy page name to buffer.
			strcpy(buff, *m_pItemList[m_pDupeItems[j].offset].page);
			// Set text only.
			lvi.mask = LVIF_TEXT;
			lvi.iSubItem = 1;			// Put text in column 1.
			lvi.pszText = buff;
			m_lstDupes.SetItem(&lvi);	// Update the item.
			++iItem;					// Next item index.
		}
	}
	// Now that list has been filled, get size of client area.
	m_lstDupes.GetClientRect(&rect);
	nColumnWidth = (rect.Width() / 2) - 4;
	// Set column widths accordingly.
	m_lstDupes.SetColumnWidth(0, nColumnWidth);
	m_lstDupes.SetColumnWidth(1, nColumnWidth);
	m_lstDupes.SetItemState(1, LVIS_SELECTED | LVIS_FOCUSED, LVIS_SELECTED | LVIS_FOCUSED);
	return TRUE;
}

void CWeedDlg::OnOK() 
{
	LV_ITEM lvi;
	UINT nCount, i, j;
	BOOL bKeepNone = TRUE;					// Flag for warning prompt.
	BOOL bNotAll, bOneSet;

	m_nRemoveCount = 0;						// Initialize count.
	nCount = m_lstDupes.GetItemCount();		// Get # of items in the list.
	lvi.mask = LVIF_PARAM | LVIF_STATE;		// Set structure elements
	lvi.iSubItem = 0;						// to retrieve the data we need
	lvi.stateMask = LVIS_STATEIMAGEMASK;	// for each item.
	for (i = 0; i < nCount; i++)			// Walk through the list.
	{
		lvi.iItem = i;						// Get data for current item.
		m_lstDupes.GetItem(&lvi);
		if (lvi.lParam >= 0)				// If item is a bookmark,
		{
			if (lvi.state == 0x2000)		// and item is not selected,
			{
				++m_nRemoveCount;			// mark it for removal.
				m_pDupeItems[lvi.lParam].remove = TRUE;
			}
			else							// If item is selected,
			{
				m_pDupeItems[lvi.lParam].remove = FALSE;
				bKeepNone = FALSE;			// Clear "no items to keep" flag.
			}
		}
	}
	if (bKeepNone)	// If no items were selected, give user a chance to go back.
	{
		if (AfxMessageBox(IDP_REMOVE_ALL, MB_YESNO | MB_ICONQUESTION) != IDYES)
		{
			m_lstDupes.SetFocus();
			return;
		}
	}
	else // Otherwise, make sure at least one item in each set will be kept.
	{
		// Clear flag.
		bNotAll = FALSE;
		// For each set of dupes:
		for (i = 0; i < m_nSetCount && !bNotAll; i++)
		{
			// Set flag.
			bOneSet = TRUE;
			// For each item in the set,
			for (j = m_pDupeSets[i].start; (j <= m_pDupeSets[i].end) && bOneSet; j++)
			{
				if (!m_pDupeItems[j].remove)	// If item is not marked for deletion,
					bOneSet = FALSE;			// Clear set flag.
			}
			// If we find a set that doesn't have at least one item
			// that is _not_ marked for removal, we're done searching.
			if (bOneSet)
				bNotAll = TRUE;
		}
		// If we found any sets with no items selected,
		// present an appropriate warning.
		if (bNotAll)
			if (AfxMessageBox(IDP_SET_UNMARKED,	MB_YESNO | MB_ICONQUESTION) != IDYES)
			{
				m_lstDupes.SetFocus();
				return;
			}
	}
	CDialog::OnOK();
}

void CWeedDlg::OnCancel() 
{
	m_nRemoveCount = 0;
	CDialog::OnCancel();
}

